import click
from virl.api import VIRLServer
from virl.helpers import get_cml_client


@click.command()
@click.option(
    "--id",
    "-i",
    required=True,
    help="ID for the Smart License feature to modify",
)
@click.option("--value", "-v", required=True, type=int, help="Number of licenses of this feature to use")
def update(id, value):
    """
    update the number of feature instances
    """
    server = VIRLServer()
    client = get_cml_client(server)
    licensing = client.licensing

    try:
        licensing.update_features({id: value})
    except Exception as e:
        click.secho("Failed to update features: {}".format(e), fg="red")
        exit(1)



from . import BaseCMLTest
from virl.api.plugin import _test_enable_plugins
from click.testing import CliRunner
import requests_mock
import os


class CMLGoodPluginTest(BaseCMLTest):
    def setUp(self):
        _test_enable_plugins()
        super().setUp()
        os.environ["CML_PLUGIN_PATH"] = os.path.realpath("./tests/v2/plugins_good")

    @classmethod
    def tearDownClass(cls):
        super().tearDownClass()
        os.environ.pop("CML_PLUGIN_PATH", None)
        _test_enable_plugins(enabled=False)

    def test_cmd_plugin_output(self):
        virl = self.get_virl()
        with requests_mock.Mocker() as m:
            runner = CliRunner()
            # Mock the request to return what we expect from the API.
            self.setup_mocks(m)
            result = runner.invoke(virl, ["--help"])
            self.assertEqual(0, result.exit_code)
            self.assertIn("test-cmd", result.output)

    def test_cmd_plugin(self):
        virl = self.get_virl()
        with requests_mock.Mocker() as m:
            # Mock the request to return what we expect from the API.
            self.setup_mocks(m)
            runner = CliRunner()
            result = runner.invoke(virl, ["test-cmd"])
            self.assertEqual("TEST COMMAND\n", result.output)

    def test_gen_plugin_output(self):
        virl = self.get_virl()
        with requests_mock.Mocker() as m:
            runner = CliRunner()
            # Mock the request to return what we expect from the API.
            self.setup_mocks(m)
            result = runner.invoke(virl, ["generate", "--help"])
            self.assertEqual(0, result.exit_code)
            self.assertIn("test-gen", result.output)

    def test_gen_plugin(self):
        virl = self.get_virl()
        with requests_mock.Mocker() as m:
            # Mock the request to return what we expect from the API.
            self.setup_mocks(m)
            runner = CliRunner()
            result = runner.invoke(virl, ["generate", "test-gen"])
            self.assertEqual("TEST GENERATOR\n", result.output)

    def test_view_plugin(self):
        virl = self.get_virl()
        with requests_mock.Mocker() as m:
            # Mock the request to return what we expect from the API.
            self.setup_mocks(m)
            runner = CliRunner()
            result = runner.invoke(virl, ["ls"])
            self.assertEqual("TEST VIEWER\n", result.output)



import tabulate
import click


def console_table(consoles):
    click.secho("Here is a list of all the running consoles")
    headers = ["Node", "Console Path"]
    table = list()
    for console in consoles:
        tr = list()
        tr.append(console["node"])
        tr.append(console["console"])
        table.append(tr)
    # wrap the output in this try/except block as some terminals
    # may have problem with the 'fancy_grid'
    try:
        click.echo(tabulate.tabulate(table, headers, tablefmt="fancy_grid"))
    except UnicodeEncodeError:
        click.echo(tabulate.tabulate(table, headers, tablefmt="grid"))


def console_table1(console_entries):
    click.secho(
        """
    Here is a list of all the running consoles
    """
    )
    headers = ["Node", "IP", "Port"]
    table = list()
    for node, ip_port in console_entries.items():
        tr = list()
        tr.append(node)
        if ip_port:
            tr.append(ip_port.split(":")[0])
            tr.append(ip_port.split(":")[1])
        table.append(tr)
    # wrap the output in this try/except block as some terminals
    # may have problem with the 'fancy_grid'
    try:
        click.echo(tabulate.tabulate(table, headers, tablefmt="fancy_grid"))
    except UnicodeEncodeError:
        click.echo(tabulate.tabulate(table, headers, tablefmt="grid"))



from . import BaseCMLTest
from click.testing import CliRunner
import requests_mock


class TestCMLLs(BaseCMLTest):
    def test_cml_ls_all(self):
        with requests_mock.Mocker() as m:
            self.setup_mocks(m)
            virl = self.get_virl()
            runner = CliRunner()
            result = runner.invoke(virl, ["ls", "--all"])
            self.assertEqual(0, result.exit_code)

    def test_cml_ls(self):
        with requests_mock.Mocker() as m:
            self.setup_mocks(m)
            virl = self.get_virl()
            runner = CliRunner()
            result = runner.invoke(virl, ["ls"])
            self.assertEqual(0, result.exit_code)



import click
from virl.api import VIRLServer
from virl.helpers import (
    check_sim_running,
    remove_sim_info,
    get_cml_client,
    safe_join_existing_lab_by_title,
    safe_join_existing_lab,
    get_current_lab,
)


@click.command()
@click.option("--id", required=False, help="An existing lab ID to stop (lab-name is ignored)")
@click.option("--lab-name", "-n", "--sim-name", required=False, help="An existing lab name to stop")
def down(id=None, lab_name=None):
    """
    stop a lab
    """
    server = VIRLServer()
    client = get_cml_client(server)

    lab = None

    if id:
        lab = safe_join_existing_lab(id, client)

    if not lab and lab_name:
        lab = safe_join_existing_lab_by_title(lab_name, client)

    if not lab:
        lab_id = get_current_lab()
        if lab_id:
            lab = safe_join_existing_lab(lab_id, client)

    if lab:
        if lab.is_active():
            click.secho("Shutting down lab {} (ID: {}).....".format(lab.title, lab.id))
            lab.stop()
            click.echo(click.style("SUCCESS", fg="green"))
        else:
            click.secho("Lab with ID {} and title {} is already stopped".format(lab.id, lab.title))

    else:
        click.secho("Failed to find lab on server", fg="red")
        exit(1)


@click.command()
@click.argument("env", default="default")
@click.option("--sim-name", required=False)
def down1(sim_name=None, env="default", **kwargs):
    """
    stop a virl simulation
    """
    # by env name
    running = check_sim_running(env)
    if running:
        sim_name = running
    # by sim name
    elif sim_name:
        sim_name = sim_name
    else:
        click.secho("Could not find sim for environment {}".format(env))
        exit(1)
    server = VIRLServer()
    resp = server.stop_simulation(sim_name)
    remove_sim_info(env=env)
    if not resp.ok:
        cause = resp.json()["cause"]
        result = click.style(cause, fg="red")
    else:
        result = click.style(resp.text, fg="green")
    click.secho("Shutting Down Simulation {}.....".format(sim_name)),
    click.echo(result)



import click
import requests


def do_pull(repo, fname, branch="master", recurse=False):
    click.secho("Pulling from {} on branch {}".format(repo, branch))
    url = "https://raw.githubusercontent.com/"
    url = url + "{}/{}/{}".format(repo, branch, fname)
    resp = requests.get(url)
    if resp.ok:
        with open(fname, "w") as fh:
            fh.write(resp.text)
        click.secho("Saved topology as {}".format(fname), fg="green")
        return True
    elif resp.status_code == 404 and not recurse:
        return do_pull(repo, fname, "main", True)
    else:
        click.secho("Error pulling {} - repo or file not found".format(repo), fg="red")
        return False


@click.command()
@click.argument("repo")
def pull(repo, **kwargs):
    """
    pull topology.yaml from repo
    """
    ret = do_pull(repo, "topology.yaml")
    if not ret:
        ret = do_pull(repo, "topology.virl")
        if not ret:
            exit(1)


@click.command()
@click.argument("repo")
def pull1(repo, **kwargs):
    """
    pull topology.virl from repo
    """
    ret = do_pull(repo, "topology.virl")
    if not ret:
        exit(1)



from . import BaseTest
from .mocks.github import MockGitHub
from click.testing import CliRunner
import requests_mock
from virl.cli.main import virl


class Tests(BaseTest):
    def test_virl_pull(self):
        with requests_mock.mock() as m:
            # Mock the request to return what we expect from the API.

            topo_url = "https://raw.githubusercontent.com/"
            topo_url += "foo/bar/master/topology.virl"
            m.get(topo_url, json=MockGitHub.get_topology())
            runner = CliRunner()
            result = runner.invoke(virl, ["pull", "foo/bar"])
            self.assertEqual(0, result.exit_code)

    def test_virl_pull_invalid_repo(self):
        with requests_mock.mock() as m:
            # Mock the request to return what we expect from the API.

            topo_url = "https://raw.githubusercontent.com/"
            topo_url += "doesnt/exist/master/topology.virl"
            m.get(topo_url, status_code=400)
            runner = CliRunner()
            result = runner.invoke(virl, ["pull", "doesnt/exist"])
            expected = "Pulling from doesnt/exist on branch master\nError pulling " "doesnt/exist - repo or file not found\n"
            self.assertEqual(result.output, expected)



from . import BaseTest
from .mocks.api import MockVIRLServer
from click.testing import CliRunner
import requests_mock
from virl.cli.main import virl

try:
    from unittest.mock import patch
except ImportError:
    from mock import patch


class Tests(BaseTest):
    @patch("virl.cli.ssh.commands.call", autospec=False)
    def test_virl_ssh(self, call_mock):

        with requests_mock.mock() as m:
            # Mock the request to return what we expect from the API.
            m.get("http://localhost:19399/roster/rest/", json=MockVIRLServer.get_sim_roster())
            runner = CliRunner()
            runner.invoke(virl, ["ssh", "router1"])
            call_mock.assert_called_once_with(["ssh", "cisco@1.1.1.1"])

    @patch("virl.cli.ssh.commands.call", autospec=False)
    def test_virl_ssh_proxy(self, call_mock):

        with requests_mock.mock() as m:
            # Mock the request to return what we expect from the API.
            m.get("http://localhost:19399/roster/rest/", json=MockVIRLServer.get_sim_roster())
            runner = CliRunner()
            runner.invoke(virl, ["ssh", "via-lxc"])
            cmd = 'ssh -o "ProxyCommand ssh -W %h:%p' ' guest@FAKE_EXTERNAL_IP" cisco@10.5.5.5'
            call_mock.assert_called_once_with(cmd, shell=True)

    @patch("virl.cli.ssh.commands.call", autospec=False)
    def test_virl_ssh_env(self, call_mock):
        with requests_mock.mock() as m:
            # Mock the request to return what we expect from the API.
            m.get("http://localhost:19399/roster/rest/", json=MockVIRLServer.get_sim_roster())
            runner = CliRunner()
            runner.invoke(virl, ["ssh", "TEST_ENV", "router1"])



import click
from virl.api import VIRLServer
from virl.helpers import get_cml_client, safe_join_existing_lab, get_current_lab


@click.command()
@click.option("--force/--no-force", "-f", default=False, required=False, help="Stop a lab (if it's started) then wipe it (default: False)")
@click.option(
    "--confirm/--no-confirm",
    show_default=False,
    default=True,
    help="Do not prompt for confirmation (default: prompt)",
    required=False,
)
def lab(force, confirm):
    """
    wipe a lab
    """
    server = VIRLServer()
    client = get_cml_client(server)

    current_lab = get_current_lab()
    if current_lab:
        lab = safe_join_existing_lab(current_lab, client)
        if lab:
            active = lab.is_active()
            if active and force:
                lab.stop(wait=True)

            # Check again just to be sure.
            if not lab.is_active():
                ret = "y"
                if confirm:
                    ret = input("Are you sure you want to wipe lab {} (ID: {}) [y/N]? ".format(lab.title, current_lab))
                if ret.lower().startswith("y"):
                    lab.wipe(wait=True)
                    click.secho("Lab {} (ID: {}) wiped".format(lab.title, current_lab))
                else:
                    click.secho("Not wiping lab {} (ID: {})".format(lab.title, current_lab))

            else:
                click.secho("Lab {} (ID: {}) is active; either down it or use --force".format(lab.title, current_lab), fg="red")
                exit(1)
        else:
            click.secho("Unable to find lab {}".format(current_lab), fg="red")
            exit(1)
    else:
        click.secho("Current lab is not set", fg="red")
        exit(1)



import click
from virl.cli.definitions.images.ls.commands import ls
from virl.cli.definitions.images.iimport import iimport
from virl.cli.definitions.images.export.commands import export


@click.group()
def images():
    """
    manage image definitions
    """
    pass


images.add_command(ls)
images.add_command(export)
images.add_command(iimport, name="import")



from pkgutil import iter_modules
from importlib import import_module
from abc import ABC, abstractmethod
import sys
import os
import click

_plugins_enabled = True


class NoPluginError(Exception):
    pass


class Plugin(ABC):
    _command_plugins = {}
    _generator_plugins = {}
    _viewer_plugins = {}

    _plugin_map = {
        "command": _command_plugins,
        "generator": _generator_plugins,
        "viewer": _viewer_plugins,
    }

    _plugin_types = [
        "CommandPlugin",
        "GeneratorPlugin",
        "ViewerPlugin",
    ]

    def __new__(cls, **kwargs):
        # only provide a plugin if global plugin support is enabled.
        if not _plugins_enabled:
            raise NoPluginError("plugin support is disabled")

        for t, d in cls._plugin_map.items():
            if t in kwargs:
                val = kwargs.pop(t)
                if val in d:
                    return object.__new__(d[val])
                else:
                    raise NoPluginError("no {} plugin for {}".format(t, val))

        raise ValueError("unsupported plugin")

    def __init_subclass__(cls, **kwargs):
        ptype = None
        pdict = None
        good_plugin = False
        for t, d in cls._plugin_map.items():
            nptype = kwargs.pop(t, None)
            if nptype and ptype:
                raise ValueError("plugin may only contain one type: {}".format(", ".join(cls._plugin_map.keys())))

            if nptype:
                ptype = nptype
                pdict = d

            if ptype:
                good_plugin = True

                if ptype not in pdict:
                    pdict[ptype] = cls

        if cls.__name__ not in cls._plugin_types and not good_plugin:
            raise ValueError("invalid plugin {}".format(cls.__name__))

    @classmethod
    def get_plugins(cls, t):
        return list(cls._plugin_map[t].keys())

    @classmethod
    def remove_plugin(cls, t, name):
        cls._plugin_map[t].pop(name, None)


class CommandPlugin(Plugin, ABC):
    def __init__(self, **kwargs):
        self._command = kwargs.pop("command")

    @property
    def command(self):
        return self._command

    @staticmethod
    @abstractmethod
    @click.command()
    def run():
        """
        This must be a "click" command.
        """
        raise NotImplementedError


class GeneratorPlugin(Plugin, ABC):
    def __init__(self, **kwargs):
        self._generator = kwargs.pop("generator")

    @property
    def generator(self):
        return self._generator

    @staticmethod
    @abstractmethod
    @click.command()
    def generate():
        """
        This must be a "click" command.
        """
        raise NotImplementedError


class ViewerPlugin(Plugin, ABC):
    def __init__(self, **kwargs):
        self._viewer = kwargs.pop("viewer")

    @property
    def viewer(self):
        return self._viewer

    @abstractmethod
    def visualize(self, **kwargs):
        raise NotImplementedError


def load_plugins(basedirs):
    if isinstance(basedirs, str):
        basedirs = basedirs.split(os.pathsep)

    modules = iter_modules(path=basedirs)
    for d in basedirs:
        if os.path.isdir:
            sys.path.append(d)

    for mod in modules:
        try:
            import_module(name=mod.name)
        except (AttributeError, ImportError, ValueError, TypeError) as e:
            # This is not a valid plugin
            click.secho(str(e), fg="red")


def check_valid_plugin(pl, mtd, mtd_name, is_click=True):
    if is_click:
        if not hasattr(mtd, "hidden") or not isinstance(pl.__class__.__dict__[mtd_name], staticmethod):
            return False

    return True


def _test_enable_plugins(enabled=True):
    """
    This function allows the unit tests to toggle
    plugin support on and off.  Without it, once
    pugins are loaded, they remain loaded for the whole
    test suite.  This can break subsequent tests.
    """
    global _plugins_enabled
    _plugins_enabled = enabled



from . import BaseTest
from .mocks.github import MockGitHub  # noqa
from click.testing import CliRunner
import requests_mock
from virl.cli.main import virl
import os

try:
    from unittest.mock import patch
except ImportError:
    from mock import patch  # noqa


class Tests(BaseTest):
    def setUp(self):
        os.environ["VIRL_HOST"] = "localhost"
        os.environ["VIRL_USERNAME"] = "guest"
        os.environ["VIRL_PASSWORD"] = "guest"

        try:
            os.remove(".virl/default/id")
            os.system("cp tests/v1/static/fake_repo_topology.virl topology.virl")

        except OSError:
            pass

    def test_01_virl_up(self):

        try:
            os.remove(".virl/default/id")
        except OSError:
            pass

        with requests_mock.mock() as m:
            # Mock the request to return what we expect from the API.
            up_url = "http://localhost:19399/simengine/rest/launch"
            m.post(up_url, json=self.mock_up_response())
            net_url = "http://localhost:19399/openstack/rest/networks"
            m.get(net_url, json=self.mock_os_net_response())
            runner = CliRunner()
            result = runner.invoke(virl, ["up"])
            self.assertEqual(0, result.exit_code)

    @patch("virl.cli.up.commands.call", autospec=False)
    def test_02_virl_up_from_repo(self, call_mock):

        try:
            os.remove(".virl/default/id")
            os.remove("topology.virl")
        except OSError:
            pass

        with requests_mock.mock() as m:
            # Mock the request to return what we expect from the API.
            up_url = "http://localhost:19399/simengine/rest/launch"
            m.post(up_url, json=self.mock_up_response())
            net_url = "http://localhost:19399/openstack/rest/networks"
            m.get(net_url, json=self.mock_os_net_response())
            topo_url = "https://raw.githubusercontent.com/"
            topo_url += "foo/bar/master/topology.virl"
            m.get(topo_url, json=MockGitHub.get_topology())
            runner = CliRunner()
            runner.invoke(virl, ["up", "foo/bar"])

    def mock_up_response(self):
        response = u"TEST_ENV"
        return response

    def mock_os_net_response(self):
        response = [
            {
                u"Description": u"L3 SNAT External",
                u"ID": u"e565b39a-ee48-4d35-b22f-15ca13cca40d",
                u"DNS": [u"8.8.8.8", u"171.70.168.183"],
                u"CIDR": u"10.10.241.208/28",
                u"Gateway": u"10.10.241.209",
                u"Network Name": u"ext-net",
            },
            {
                u"Description": u"L2 FLAT",
                u"ID": u"426a32c0-4964-4288-ba74-7db1f1f0b848",
                u"DNS": [u"8.8.8.8", u"171.70.168.183"],
                u"CIDR": u"10.10.241.224/27",
                u"Gateway": u"10.10.241.225",
                u"Network Name": u"flat",
            },
            {
                u"Description": u"L2 FLAT",
                u"ID": u"bfa4c58b-e404-45af-905d-b50aa603df20",
                u"DNS": [u"8.8.8.8", u"171.70.168.183"],
                u"CIDR": u"172.16.1.192/27",
                u"Gateway": u"172.16.1.193",
                u"Network Name": u"flat1",
            },
        ]
        return response



import tabulate
import click


def repo_table(repo_entries):

    # sort by date
    headers = ["Name", "Stars", "Description"]
    table = list()

    for repo in repo_entries:
        tr = list()
        tr.append(repo['full_name'])
        tr.append(repo['stargazers_count'])
        tr.append(repo['description'])
        table.append(tr)
    # wrap the output in this try/except block as some terminals
    # may have problem with the 'fancy_grid'
    try:
        click.echo(tabulate.tabulate(table, headers, tablefmt="fancy_grid"))
    except UnicodeEncodeError:
        click.echo(tabulate.tabulate(table, headers, tablefmt="grid"))



import os
from yaml import load

try:
    from yaml import CLoader as Loader
except ImportError:
    from yaml import Loader


class CachedLab(object):
    """
    This is a stub class to represent a pseudo-lab that is cached locally.
    Only enough of the lab model is implemented for the required virlutils functions.
    """

    __id = None
    __title = None
    __description = None
    __stats = {
        "nodes": 0,
        "links": 0,
        "interfaces": 0,
    }

    def __init__(self, lab_id, cache_file):
        if not os.path.exists(cache_file):
            raise FileNotFoundError("Cached lab {} not found".format(cache_file))

        self.__id = lab_id

        with open(cache_file, "rb") as fd:
            lab = load(fd, Loader=Loader)

            self.__title = lab["lab"]["title"]
            self.__description = lab["lab"]["description"]
            self.__stats["nodes"] = len(lab["nodes"])
            self.__stats["links"] = len(lab["links"])
            for n in lab["nodes"]:
                self.__stats["interfaces"] += len(n["interfaces"])

    @property
    def id(self):
        return self.__id

    @property
    def title(self):
        return self.__title

    @property
    def description(self):
        return self.__description

    def state(self):
        return "CACHED"

    @property
    def statistics(self):
        return self.__stats

    @property
    def owner(self):
        return "N/A"



from flask import Flask
from flask import render_template
import os

app = Flask(__name__)


@app.route('/')
def swagger_ui(name=None):
    username = os.environ['VIRL_SWAGGER_USERNAME']
    password = os.environ['VIRL_SWAGGER_PASSWORD']
    return render_template('index.html', username=username, password=password)


@app.route('/swagger.json')
def swagger_def(name=None):
    host = os.environ['VIRL_SWAGGER_HOST']
    port = os.environ['VIRL_SWAGGER_PORT']
    return render_template('swagger.json', host=host, port=port)



import click
from virl.api import VIRLServer, ViewerPlugin, NoPluginError
from subprocess import call
from virl import helpers
from virl.helpers import get_cml_client, get_current_lab, safe_join_existing_lab, get_command
from virl.cli.views.console import console_table, console_table1
from virl2_client.exceptions import NodeNotFound
import platform


@click.command()
@click.argument("node", nargs=1)
@click.option("--display/--none", default="False", help="Display Console information")
def console(node, display, **kwargs):
    """
    console for node
    """
    server = VIRLServer()
    client = get_cml_client(server)
    skip_types = ["external_connector", "unmanaged_switch"]

    current_lab = get_current_lab()
    if current_lab:
        lab = safe_join_existing_lab(current_lab, client)
        if lab:
            try:
                node_obj = lab.get_node_by_label(node)
            except NodeNotFound:
                click.secho("Node {} was not found in lab {}".format(node, current_lab), fg="red")
                exit(1)

            if node_obj.node_definition not in skip_types:
                if node_obj.is_active():
                    console = "/{}/{}/0".format(lab.id, node_obj.id)
                    if display:
                        try:
                            pl = ViewerPlugin(viewer="console")
                            pl.visualize(consoles=[{"node": node, "console": console}])
                        except NoPluginError:
                            console_table([{"node": node, "console": console}])
                    else:
                        # use user specified ssh command
                        if "CML_CONSOLE_COMMAND" in server.config:
                            cmd = server.config["CML_CONSOLE_COMMAND"]
                            cmd = cmd.format(host=server.host, user=server.user, console="open " + console)
                            print("Calling user specified command: {}".format(cmd))
                            exit(call(cmd.split()))

                        # someone still uses windows
                        elif platform.system() == "Windows":
                            with helpers.disable_file_system_redirection():
                                cmd = "ssh -t {}@{} open {}".format(server.user, server.host, console)
                                exit(call(cmd.split()))

                        # why is shit so complicated?
                        else:
                            cmd = "ssh -t {}@{} open {}".format(server.user, server.host, console)
                            exit(call(cmd.split()))
                else:
                    click.secho("Node {} is not active".format(node), fg="red")
                    exit(1)
            else:
                click.secho("Node type {} does not support console connectivity".format(node_obj.node_definition), fg="yellow")
        else:
            click.secho("Unable to find lab {}".format(current_lab), fg="red")
            exit(1)
    else:
        click.secho("No current lab set", fg="red")
        exit(1)


@click.command()
@click.argument("node", nargs=-1)
@click.option("--display/--none", default="False", help="Display Console information")
def console1(node, display, **kwargs):
    """
    console for node
    """
    server = VIRLServer()

    if len(node) == 2:
        # we received env and node name
        env = node[0]
        running = helpers.check_sim_running(env)
        node = node[1]
    elif display:
        # only displaying output
        env = "default"
        running = helpers.check_sim_running(env)
        node = None

    elif len(node) == 1:
        # assume default env
        env = "default"
        running = helpers.check_sim_running(env)
        node = node[0]
    else:
        # node was not specified, display usage
        exit(call([get_command(), "console", "--help"]))

    if running:

        sim_name = running

        resp = server.get_node_console(sim_name, node=node)
        if node:
            click.secho("Attempting to connect to console " "of {}".format(node))
            try:
                ip, port = resp.json()[node].split(":")

                # use user specified telnet command
                if "VIRL_CONSOLE_COMMAND" in server.config:
                    cmd = server.config["VIRL_CONSOLE_COMMAND"]
                    cmd = cmd.format(host=ip, port=port)
                    print("Calling user specified command: {}".format(cmd))
                    exit(call(cmd.split()))

                # someone still uses windows
                elif platform.system() == "Windows":
                    with helpers.disable_file_system_redirection():
                        exit(call(["telnet", ip, port]))

                # why is shit so complicated?
                else:
                    exit(call(["telnet", ip, port]))
            except AttributeError:
                click.secho("Could not find console info for " "{}:{}".format(env, node), fg="red")
            except KeyError:
                click.secho("Unknown node {}:{}".format(env, node), fg="red")
        else:
            # defaults to displaying table
            console_table1(resp.json())



from . import BaseTest
from click.testing import CliRunner
from virl.cli.main import virl


class TestHelp(BaseTest):

    def test_virl_help(self):
        runner = CliRunner()
        result = runner.invoke(virl, ["--help"])
        self.assertEqual(0, result.exit_code)
        for command in ["console", "generate", "down", "nodes", "logs",
                        "ls", "pull", "search", "ssh", "start", "stop",
                        "telnet", "up", "use"]:
            self.assertIn(command, result.output)



import click
import tabulate


def image_list_table(image_list):

    headers = ["ID", "Node Definition ID", "Label", "Description", "RAM", "CPUs", "Boot Disk Size"]
    table = list()

    for f in list(image_list):
        tr = list()
        tr.append(str(f["id"]))
        tr.append(str(f["node_definition_id"]))
        tr.append(str(f["label"]))
        tr.append(str(f["description"]))
        tr.append(str(f["ram"]))
        tr.append(str(f["cpus"]))
        tr.append(str(f["boot_disk_size"]))

        table.append(tr)

    try:
        click.echo(tabulate.tabulate(table, headers, tablefmt="fancy_grid"))
    except UnicodeEncodeError:
        click.echo(tabulate.tabulate(table, headers, tablefmt="grid"))



import click
from virl.api import VIRLServer
from virl.helpers import get_env_sim_name, get_cml_client, safe_join_existing_lab, get_current_lab, extract_configurations


@click.command()
@click.option("--extract/--no-extract", default=True, help="extract the configurations from devices before export (default: True)")
@click.option(
    "-f", "--filename", required=False, default="topology.yaml", metavar="<filename>", help="filename to save to, defaults to topology.yaml"
)
def save(extract, filename, **kwargs):
    """
    save lab to a local yaml file
    """
    server = VIRLServer()
    client = get_cml_client(server)

    current_lab = get_current_lab()
    if current_lab:
        lab = safe_join_existing_lab(current_lab, client)
        if lab:
            if extract:
                click.secho("Extracting configurations...")
                extract_configurations(lab)

            lab_export = lab.download()

            click.secho("Writing {}".format(filename))
            with open(filename, "w") as fd:
                fd.write(lab_export)
        else:
            click.secho("Failed to find running lab {}".format(current_lab), fg="red")
            exit(1)
    else:
        click.secho("Current lab is not set", fg="red")
        exit(1)


@click.command()
@click.argument("env", default="default")
@click.option("--ip/--no-ip", default=False, help="include dynamically assigned addresses")
@click.option(
    "-f", "--filename", required=False, default="topology.virl", metavar="<filename>", help="filename to save to, defaults to topology.virl"
)
def save1(env, ip, filename, **kwargs):
    """
    save simulation to local virl file
    """
    with open(filename, "w") as fh:
        sim_name = get_env_sim_name(env)
        server = VIRLServer()
        resp = server.export(sim_name, ip=ip)
        click.secho("Saving {} to {}".format(sim_name, filename))
        fh.write(resp.text)



from collections import OrderedDict
import yaml
from jinja2 import Environment, PackageLoader
from lxml import etree
from virl.helpers import get_node_mgmt_ip


def setup_yaml():
    """ https://stackoverflow.com/a/8661021 """
    represent_dict_order = lambda self, data: self.represent_mapping("tag:yaml.org,2002:map", data.items())  # noqa
    yaml.add_representer(OrderedDict, represent_dict_order)


setup_yaml()


def create_group_map(virl_xmlstr):
    """
    retrieves ansible group information from `virl_xmlstr`
    returns a dictionary handy for determining the group that a node belongs to

    NOTE:  `virl_xmlstr` must be received from the VIRL API, not the virl file

    Ansible Groups are controlled by adding an extension to the VIRL XML file

    e.g

    <node name="router1" type="SIMPLE" subtype="CSR1000v">
        <extensions>
            <entry key="ansible_group" type="String">mygroup</entry>
        </extensions>
     </node>

     """
    root = etree.fromstring(virl_xmlstr)

    # get all ansible_groups
    groups = root.xpath("//virl:entry[@key='ansible_group']/text()", namespaces={"virl": "http://www.cisco.com/VIRL"})
    groups = set(groups)
    group_map = dict()

    # populate nodes in correct groups
    for g in groups:
        # query returns a list strings representing the node names in group
        q = '//virl:node//virl:entry[text()="{}"]/ancestor::virl:node/@name'
        query = q.format(g)
        members = root.xpath(query, namespaces={"virl": "http://www.cisco.com/VIRL"})
        # add to map so that hostnames can be resolved to groups later
        for m in members:
            group_map[m] = g

    return group_map


def generate_inventory_dict1(virl_xml, roster=None, interfaces=None):
    """
    common inventory info accross yaml/ini
    """
    # create inventory skeleton
    inventory = dict()
    inventory["all"] = dict()
    inventory["all"]["children"] = dict()
    inventory["all"]["hosts"] = dict()

    for node_long_name, device in roster.items():
        # make sure we have at least enough info to be useful
        if ("managementIP") in device and ("NodeName" in device):
            entry = dict()
            entry["node_long_name"] = node_long_name
            entry["ansible_host"] = device["managementIP"]
        # otherwise, skip this device
        else:
            continue

        name = device["NodeName"]
        # map console ports if they are available
        if all(k in device for k in ("SimulationHost", "PortConsole")):
            entry["console_server"] = device["SimulationHost"]
            entry["console_port"] = device["PortConsole"]

        # determine device/os type
        try:
            type = device["NodeSubtype"]
            if "NX" in type:
                entry["device_type"] = "nxos"
            elif "XR" in type:
                entry["device_type"] = "iosxr"
            elif "CSR" in type:
                entry["device_type"] = "ios"
            elif "IOS" in type:
                entry["device_type"] = "ios"

            else:
                entry["device_type"] = "unknown"

        except KeyError:
            entry["device_type"] = "unknown"

        # create data structure for resolving group name from node name
        group_map = create_group_map(virl_xml)
        # add group to children

        # try to map to ansible group
        try:
            group = group_map[name]
            print("Placing {} into ansible group {}".format(name, group))
            if group not in inventory["all"]["children"]:
                inventory["all"]["children"][group] = dict()
            if name not in inventory["all"]["children"]:
                inventory["all"]["children"][group][name] = entry

        # otherwise it will go in all:
        except:  # noqa
            inventory["all"]["hosts"][name] = entry

    # try:
    #     devices = interfaces[sim_name]
    # except KeyError:
    #     raise Exception('something went wrong')

    return inventory


def generate_inventory_dict(lab, server):
    """
    common inventory info accross yaml/ini
    """
    # create inventory skeleton
    inventory = dict()
    inventory["all"] = dict()
    inventory["all"]["children"] = dict()
    inventory["all"]["hosts"] = dict()

    for node in lab.nodes():
        mgmtip = get_node_mgmt_ip(node)

        if not mgmtip:
            continue

        name = node.label
        entry = dict()
        entry["ansible_host"] = mgmtip
        # map console ports if they are available
        entry["console_server"] = server.host
        entry["console_user"] = server.user
        entry["console_path"] = "/{}/{}/0".format(lab.id, node.id)

        # determine device/os type
        try:
            type = node.node_definition.lower()
            if "nx" in type:
                entry["device_type"] = "nxos"
            elif "xr" in type:
                entry["device_type"] = "iosxr"
            elif "csr" in type:
                entry["device_type"] = "ios"
            elif "ios" in type:
                entry["device_type"] = "ios"
            elif "asa" in type:
                entry["device_type"] = "asa"
            else:
                entry["device_type"] = "unknown"

        except KeyError:
            entry["device_type"] = "unknown"

        ansible_group = None
        for tag in node.tags():
            if tag.startswith("ansible_group="):
                ansible_group = tag.split("=")[1].strip()
                break

        # try to map to ansible group
        if ansible_group:
            print("Placing {} into ansible group {}".format(name, ansible_group))
            if ansible_group not in inventory["all"]["children"]:
                inventory["all"]["children"][ansible_group] = dict()
            if name not in inventory["all"]["children"]:
                inventory["all"]["children"][ansible_group][name] = entry
        else:
            inventory["all"]["hosts"][name] = entry

    return inventory


def render_yaml_inventory1(virl_xml, roster=None, interfaces=None):
    """
    we need to merge information from multiple sources to generate all
    the required parameters for the inventory

    """

    j2_env = Environment(loader=PackageLoader("virl"), trim_blocks=False)

    inventory = generate_inventory_dict1(virl_xml, roster=roster, interfaces=interfaces)
    template = j2_env.get_template("ansible/inventory_template1.j2")
    return template.render(inventory=inventory)


def render_ini_inventory1(virl_xml, roster=None, interfaces=None):
    """
    we need to merge information from multiple sources to generate all
    the required parameters for the inventory

    """

    j2_env = Environment(loader=PackageLoader("virl"), trim_blocks=False)

    inventory = generate_inventory_dict1(virl_xml, roster=roster, interfaces=interfaces)
    # pass all available data to template for rendering, this can probably
    # be pruned back at some point
    template = j2_env.get_template("ansible/inventory_ini_template1.j2")
    return template.render(inventory=inventory)


def render_inventory(lab, server, style):
    """
    render the YAML version of the inventory
    """

    j2_env = Environment(loader=PackageLoader("virl"), trim_blocks=False)

    inventory = generate_inventory_dict(lab, server)
    template = None
    if style == "ini":
        template = j2_env.get_template("ansible/inventory_ini_template.j2")
    elif style == "yaml":
        template = j2_env.get_template("ansible/inventory_template.j2")

    if template:
        return template.render(inventory=inventory, lab_id=lab.id, lab_title=lab.title)

    return None


def ansible_inventory_generator1(env, virl_data, roster, interfaces, style="yaml"):
    """
    given a sim roster produces a inventory file suitable for
    use with ansible

    """
    if not all([virl_data, roster, interfaces]):
        raise ValueError("we really need virl_xml, roster, and interfaces")

    # sim_name should be the only key in the dictionary
    if len(interfaces.keys()) != 1:
        raise ValueError("too many keys in interface response")

    if style == "yaml":
        inventory_yaml = render_yaml_inventory1(virl_data, roster, interfaces)
        return inventory_yaml
    elif style == "ini":
        inventory_ini = render_ini_inventory1(virl_data, roster, interfaces)
        return inventory_ini


def ansible_inventory_generator(lab, server, style="yaml"):
    """
    given a lab produces an inventory file suitable for use with ansible
    """
    inv = render_inventory(lab, server, style)

    return inv



import click
from virl.api import plugin, check_valid_plugin, NoPluginError
from virl.cli.generate.ansible.commands import ansible, ansible1
from virl.cli.generate.pyats.commands import pyats, pyats1
from virl.cli.generate.nso.commands import nso, nso1


@click.group()
def generate():
    """
    generate inv file for various tools
    """
    pass


@click.group()
def generate1():
    """
    generate inv file for various tools
    """
    pass


generate1.add_command(ansible1, name="ansible")
generate1.add_command(pyats1, name="pyats")
generate1.add_command(nso1, name="nso")


def init_generators():
    generate.add_command(ansible)
    generate.add_command(pyats)
    generate.add_command(nso)

    for gen in plugin.Plugin.get_plugins("generator"):
        try:
            pl = plugin.GeneratorPlugin(generator=gen)
        except NoPluginError:
            continue
        if not check_valid_plugin(pl, pl.generate, "generate"):
            click.secho(
                "ERROR: Malformed plugin for generator {}.  The `generate` method must be static and a click.command".format(gen), fg="red"
            )
            plugin.Plugin.remove_plugin("generator", gen)
        else:
            generate.add_command(pl.generate, name=gen)



from . import BaseTest
from .mocks import MockVIRLServer
from click.testing import CliRunner
import requests_mock
from virl.cli.main import virl

try:
    from unittest.mock import patch
except ImportError:
    from mock import patch


class Tests(BaseTest):
    def test_virl_console_display(self):
        with requests_mock.mock() as m:
            # Mock the request to return what we expect from the API.
            m.get("http://localhost:19399/simengine/rest/serial_port/TEST_ENV", json=MockVIRLServer.get_node_console())
            runner = CliRunner()
            result = runner.invoke(virl, ["console", "--display"])
            self.assertEqual(0, result.exit_code)

    @patch("virl.cli.console.commands.call", autospec=False)
    def test_virl_console_connect(self, call_mock):

        with requests_mock.mock() as m:
            # Mock the request to return what we expect from the API.
            m.get("http://localhost:19399/simengine/rest/serial_port/TEST_ENV", json=MockVIRLServer.get_node_console())
            runner = CliRunner()
            runner.invoke(virl, ["console", "router1"])
            call_mock.assert_called_once_with(["telnet", u"10.94.241.194", u"17001"])

    @patch("virl.cli.console.commands.call", autospec=False)
    def test_virl_console_connect_env(self, call_mock):

        with requests_mock.mock() as m:
            # Mock the request to return what we expect from the API.
            m.get("http://localhost:19399/simengine/rest/serial_port/TEST_ENV", json=MockVIRLServer.get_node_console())
            runner = CliRunner()
            runner.invoke(virl, ["console", "TEST_ENV", "router1"])



import click
from virl.api import VIRLServer
from subprocess import call
from virl import helpers
from virl.helpers import get_mgmt_lxc_ip, get_node_from_roster, get_cml_client, get_current_lab, safe_join_existing_lab, get_node_mgmt_ip
from virl2_client.exceptions import NodeNotFound


@click.command()
@click.argument("node", nargs=1)
def ssh(node):
    """
    ssh to a node
    """
    server = VIRLServer()
    client = get_cml_client(server)
    username = server.config.get("VIRL_SSH_USERNAME", "cisco")

    current_lab = get_current_lab()
    if current_lab:
        lab = safe_join_existing_lab(current_lab, client)
        if lab:
            try:
                node_obj = lab.get_node_by_label(node)
            except NodeNotFound:
                click.secho("Node {} was not found in lab {}".format(node, current_lab), fg="red")
                exit(1)

            if node_obj.is_active():
                mgmtip = get_node_mgmt_ip(node_obj)
                if mgmtip:
                    if "VIRL_SSH_COMMAND" in server.config:
                        cmd = server.config["VIRL_SSH_COMMAND"]
                        cmd = cmd.format(host=mgmtip, username=username)
                        print("Calling user specified command: {}".format(cmd))
                        exit(call(cmd.split()))
                    else:
                        click.secho("Attemping ssh connection to {} at {}".format(node_obj.label, mgmtip))

                        exit(call(["ssh", "{}@{}".format(username, mgmtip)]))
                else:
                    click.secho("Node {} does not have an external management IP".format(node_obj.label))
            else:
                click.secho("Node {} is not active".format(node_obj.label), fg="yellow")
        else:
            click.secho("Unable to find lab {}".format(current_lab), fg="red")
            exit(1)
    else:
        click.secho("No current lab set", fg="red")
        exit(1)


@click.command()
@click.argument("node", nargs=-1)
def ssh1(node):
    """
    ssh to a node
    """
    if len(node) == 2:
        # we received env and node name
        env = node[0]
        running = helpers.check_sim_running(env)
        node = node[1]
    elif len(node) == 1:
        # assume default env
        env = "default"
        running = helpers.check_sim_running(env)
        node = node[0]
    else:
        exit(call(["virl", "ssh", "--help"]))

    if running:
        sim_name = running
        server = VIRLServer()
        details = server.get_sim_roster(sim_name)

        # default ssh username can be overriden
        username = server.config.get("VIRL_SSH_USERNAME", "cisco")

        if node:
            try:
                node_dict = get_node_from_roster(node, details)
                node_name = node_dict.get("NodeName")
                ip = node_dict["managementIP"]
                proxy = node_dict.get("managementProxy")

                if "VIRL_SSH_COMMAND" in server.config:
                    cmd = server.config["VIRL_SSH_COMMAND"]
                    cmd = cmd.format(host=ip, username=username)
                    print("Calling user specified command: {}".format(cmd))
                    exit(call(cmd.split()))

                if proxy == "lxc":
                    lxc = get_mgmt_lxc_ip(details)
                    if lxc:
                        click.secho("Attemping ssh connection" "to {} at {} via {}".format(node_name, ip, lxc))
                        cmd = 'ssh -o "ProxyCommand ssh -W %h:%p {}@{}" {}@{}'
                        cmd = cmd.format(server.user, lxc, username, ip)

                        exit(call(cmd, shell=True))
                else:
                    # handle the "flat" networking case
                    click.secho("Attemping ssh connection" "to {} at {}".format(node_name, ip))

                    exit(call(["ssh", "{}@{}".format(username, ip)]))

            except AttributeError:
                click.secho("Could not find management info" " for {}:{}".format(env, node), fg="red")

            except KeyError:
                click.secho("Unknown node {}:{}".format(env, node), fg="red")
        else:
            return details.json()



import click
from virl.cli.license.show.commands import show as showc
from virl.cli.license.register.commands import register as registerc
from virl.cli.license.renew import renew as renewc
from virl.cli.license.deregister.commands import deregister as deregisterc
from virl.cli.license.features import features as featuresc


@click.group()
def license():
    """
    work with product licensing
    """
    pass


license.add_command(showc, name="show")
license.add_command(registerc, name="register")
license.add_command(renewc, name="renew")
license.add_command(deregisterc, name="deregister")
license.add_command(featuresc, name="features")



from . import BaseTest
from click.testing import CliRunner
import requests_mock
from virl.cli.main import virl

try:
    from unittest.mock import patch
except ImportError:
    from mock import patch


class Tests(BaseTest):
    def test_virl_stop(self):
        with requests_mock.mock() as m:
            # Mock the request to return what we expect from the API.
            down_url = "http://localhost:19399/simengine/rest/stop/TEST_ENV"
            m.get(down_url, json=self.mock_down_response())
            runner = CliRunner()
            runner.invoke(virl, ["stop", "router1"])

    def test_virl_start_env(self):
        with requests_mock.mock() as m:
            # Mock the request to return what we expect from the API.
            down_url = "http://localhost:19399/simengine/rest/start/foo_env"
            m.get(down_url, json=self.mock_down_response())
            runner = CliRunner()
            runner.invoke(virl, ["stop", "foo_env", "router1"])

    @patch("virl.cli.stop.commands.call", autospec=False)
    def test_virl_stop_no_node_given(self, call_mock):
        with requests_mock.mock() as m:
            # Mock the request to return what we expect from the API.
            down_url = "http://localhost:19399/simengine/rest/stop/TEST_ENV"
            m.get(down_url, json=self.mock_down_response())
            runner = CliRunner()
            runner.invoke(virl, ["stop"])
            call_mock.assert_called_with(["virl", "stop", "--help"])

    def mock_down_response(self):
        response = "SUCCESS"
        return response



import click

from virl.api import VIRLServer
from virl.cli.views import log_table
from virl import helpers


@click.command()
@click.argument('env', default='default')
def logs1(env, **kwargs):
    """
    Retrieves log information for the provided simulation
    """
    running = helpers.check_sim_running(env)
    if running:
        sim_name = running
        server = VIRLServer()
        resp = server.get_logs(sim_name)
        log_table(resp.json()['events'])
    else:
        click.secho("could not find logs for for env: {}".format(env),
                    fg='red')



from . import BaseCMLTest
from virl.api.plugin import _test_enable_plugins
from click.testing import CliRunner
import requests_mock
import os

try:
    from unittest.mock import patch
except ImportError:
    from mock import patch  # noqa


class CMLBadPluginTest(BaseCMLTest):
    def localSetUp(self, pdir):
        os.environ["CML_PLUGIN_PATH"] = os.path.realpath("./tests/v2/{}".format(pdir))
        _test_enable_plugins()

    def tearDown(self):
        super().tearDown()
        os.environ.pop("CML_PLUGIN_PATH", None)

    @classmethod
    def tearDownClass(cls):
        super().tearDownClass()
        os.environ.pop("CML_PLUGIN_PATH", None)
        _test_enable_plugins(enabled=False)

    @patch("virl.cli.main.click.secho", autospec=False)
    def test_cmd_plugin_bad(self, secho_mock):
        self.localSetUp("plugins_bad_cmd")
        virl = self.get_virl()
        with requests_mock.Mocker() as m:
            runner = CliRunner()
            # Mock the request to return what we expect from the API.
            self.setup_mocks(m)
            result = runner.invoke(virl, ["--help"])
            self.assertEqual(0, result.exit_code)
            self.assertNotIn("test-bad-cmd", result.output)
            secho_mock.assert_called_once_with(
                "ERROR: Malformed plugin for command test-bad-cmd.  The `run` method must be static and a click.command", fg="red"
            )

    @patch("virl.cli.generate.click.secho", autospec=False)
    def test_gen_plugin_bad(self, secho_mock):
        self.localSetUp("plugins_bad_gen")
        virl = self.get_virl()
        with requests_mock.Mocker() as m:
            runner = CliRunner()
            # Mock the request to return what we expect from the API.
            self.setup_mocks(m)
            result = runner.invoke(virl, ["generate", "--help"], catch_exceptions=False)
            self.assertEqual(0, result.exit_code)
            self.assertNotIn("test-bad-gen", result.output)
            secho_mock.assert_called_once_with(
                "ERROR: Malformed plugin for generator test-bad-gen.  The `generate` method must be static and a click.command", fg="red"
            )

    @patch("virl.api.plugin.click.secho", autospec=False)
    def test_view_plugin_bad(self, secho_mock):
        self.localSetUp("plugins_bad_viewer")
        virl = self.get_virl()
        with requests_mock.Mocker() as m:
            runner = CliRunner()
            # Mock the request to return what we expect from the API.
            self.setup_mocks(m)
            result = runner.invoke(virl, ["ls"])
            self.assertEqual(0, result.exit_code)
            self.assertNotIn("VIEWER PLUGIN", result.output)
            secho_mock.assert_any_call("invalid plugin BadLabViewer", fg="red")


"""

    def test_gen_plugin(self):
        virl = self.get_virl()
        with requests_mock.Mocker() as m:
            # Mock the request to return what we expect from the API.
            self.setup_mocks(m)
            runner = CliRunner()
            result = runner.invoke(virl, ["generate", "test-gen"])
            self.assertEqual("TEST GENERATOR\n", result.output)

    def test_view_plugin(self):
        virl = self.get_virl()
        with requests_mock.Mocker() as m:
            # Mock the request to return what we expect from the API.
            self.setup_mocks(m)
            runner = CliRunner()
            result = runner.invoke(virl, ["ls"])
            self.assertEqual("TEST VIEWER\n", result.output)
"""



import click
from virl.api import ViewerPlugin, NoPluginError
from virl.api.github import get_repos
from virl.cli.views.search import repo_table


@click.command()
@click.argument("query", required=False)
@click.option("--org", default="virlfiles", required=False, help="GitHub organization to search (default: virlfiles)")
def search(query=None, **kwargs):
    """
    list topologies available via github
    """

    repos = get_repos(org=kwargs["org"], query=query)
    if query is not None:
        click.secho("Displaying {} Results For {}".format(len(repos), query))
    else:
        click.secho("Displaying {} Results".format(len(repos)))
    try:
        pl = ViewerPlugin(viewer="search")
        pl.visualize(repos=repos)
    except NoPluginError:
        repo_table(repos)



import click
import os
from virl.api import VIRLServer
from virl.helpers import get_cml_client
from virl2_client.models.licensing import DEFAULT_SSMS


@click.command()
@click.option(
    "--token",
    "-t",
    required=True,
    help="Smart License token for registration",
)
@click.option(
    "--reregister/--no-reregister",
    "--force/--no-force",
    required=False,
    default=False,
    help="Force registration even if already registered",
)
@click.option("--smart-license-server", "-s", required=False, help="URL for the Smart License server or satellite")
@click.option("--proxy-host", "-p", required=False, help="Hostname or IP address of proxy to use for registration (if required)")
@click.option("--proxy-port", "-o", required=False, default=80, help="Port number to use for proxy host (default: 80)")
@click.option("--certificate", "-c", required=False, help="Path to a PEM-encoded certificate for the Smart License SSMS")
def register(token, **kwargs):
    """
    register with a Smart License account
    """
    ssms = kwargs["smart_license_server"]
    proxy = kwargs["proxy_host"]
    port = None
    cert = kwargs["certificate"]
    reregister = kwargs["reregister"]
    server = VIRLServer()
    client = get_cml_client(server)
    licensing = client.licensing

    if ssms or proxy:
        if not ssms:
            ssms = DEFAULT_SSMS
        if proxy:
            port = kwargs["proxy_port"]

        try:
            licensing.set_transport(ssms, proxy, port)
        except Exception as e:
            click.secho("Failed to configure Smart License server and proxy: {}".format(e), fg="red")
            exit(1)
    else:
        try:
            licensing.delete_certificate()
        except Exception:
            pass

        licensing.set_default_transport()

    if cert:
        if not os.path.isfile(cert):
            click.secho("Certificate {} is not a valid file!".format(cert), fg="red")
            exit(1)

        with open(cert, "r") as fd:
            contents = fd.read()
            try:
                licensing.upload_certificate(contents)
            except Exception as e:
                click.secho("Failed to upload certificate {}: {}".format(cert, e), fg="red")
                exit(1)

    try:
        licensing.register(token, reregister)
    except Exception as e:
        click.secho("Failed to register with Smart Licensing: {}".format(e), fg="red")
        exit(1)



from . import BaseTest
from .mocks import MockVIRLServer
from click.testing import CliRunner
import requests_mock
from virl.cli.main import virl

# import filecmp


class Tests(BaseTest):
    def test_virl_generate_pyats(self):
        with requests_mock.mock() as m:
            # Mock the request to return what we expect from the API.
            roster_url = "http://localhost:19399/roster/rest/"
            m.get(roster_url, json=MockVIRLServer.get_sim_roster())
            export_url = "http://localhost:19399/simengine/rest/export/"
            export_url += "TEST_ENV?running-configs=config&updated=true"
            m.get(export_url, text=MockVIRLServer.export())
            interface_url = "http://localhost:19399/simengine/rest/interfaces/"
            interface_url += "TEST_ENV"
            m.get(interface_url, json=MockVIRLServer.get_interfaces())
            runner = CliRunner()
            result = runner.invoke(virl, ["generate", "pyats"])
            # same = filecmp.cmp('default_testbed.yaml',
            #                    'tests/static/pyats_testbed')
            # self.assertTrue(same)
            self.assertEqual(0, result.exit_code)



from .pyats_testbed import pyats_testbed_generator1, pyats_testbed_generator  # noqa
from .ansible_inventory import ansible_inventory_generator1, ansible_inventory_generator  # noqa
from .nso_payload import nso_payload_generator1, nso_payload_generator  # noqa



from . import BaseCMLTest
from click.testing import CliRunner
import requests_mock
import os


class CMLSaveTests(BaseCMLTest):
    def test_cml_save(self):
        with requests_mock.Mocker() as m:
            # Mock the request to return what we expect from the API.
            self.setup_mocks(m)
            virl = self.get_virl()
            runner = CliRunner()
            result = runner.invoke(virl, ["save"])
            self.assertEqual(0, result.exit_code)
            self.assertIn("Extracting configurations", result.output)
            self.assertIn("Writing topology.yaml", result.output)

    def test_cml_save_no_extract(self):
        with requests_mock.Mocker() as m:
            # Mock the request to return what we expect from the API.
            self.setup_mocks(m)
            virl = self.get_virl()
            runner = CliRunner()
            result = runner.invoke(virl, ["save", "--no-extract"])
            self.assertEqual(0, result.exit_code)
            self.assertNotIn("Extracting configurations", result.output)
            self.assertIn("Writing topology.yaml", result.output)

    def test_cml_save_filename(self):
        with requests_mock.Mocker() as m:
            # Mock the request to return what we expect from the API.
            self.setup_mocks(m)
            virl = self.get_virl()
            runner = CliRunner()
            result = runner.invoke(virl, ["save", "-f", "{}.yaml".format(self.get_test_id())])
            self.assertEqual(0, result.exit_code)
            self.assertIn("Extracting configurations", result.output)
            self.assertIn("Writing {}.yaml".format(self.get_test_id()), result.output)

    def test_cml_save_no_lab(self):
        try:
            os.remove(".virl/current_cml_lab")
        except OSError:
            pass

        with requests_mock.Mocker() as m:
            # Mock the request to return what we expect from the API.
            self.setup_mocks(m)
            virl = self.get_virl()
            runner = CliRunner()
            result = runner.invoke(virl, ["save"])
            self.assertEqual(1, result.exit_code)
            self.assertIn("Current lab is not set", result.output)

    def test_cml_save_bogus_lab(self):
        try:
            os.remove(".virl/current_cml_lab")
        except OSError:
            pass

        src_dir = os.path.realpath(".virl")
        with open(".virl/cached_cml_labs/123456", "w") as fd:
            fd.write("lab: bogus\n")

        os.symlink("{}/cached_cml_labs/123456".format(src_dir), "{}/current_cml_lab".format(src_dir))

        with requests_mock.Mocker() as m:
            # Mock the request to return what we expect from the API.
            self.setup_mocks(m)
            virl = self.get_virl()
            runner = CliRunner()
            result = runner.invoke(virl, ["save"])
            os.remove(".virl/cached_cml_labs/123456")
            os.remove(".virl/current_cml_lab")
            self.assertEqual(1, result.exit_code)
            self.assertIn("Failed to find running lab 123456", result.output)



import click
from virl.api import VIRLServer
from virl import helpers
import subprocess


@click.command()
@click.argument("env", default="default")
def viz1(env):
    """
    opens live visualization for the sim
    """
    server = VIRLServer()
    running = helpers.check_sim_running(env)

    if running:
        sim_name = running
        url = "http://{}:19402/?sim_id={}#/layer/phy".format(server.host, sim_name)
        subprocess.Popen(["open", url])
    else:
        click.secho("No Running simulation to visualize", fg="red")



import os
import click
from virl.api import VIRLServer, CachedLab, ViewerPlugin, NoPluginError
from virl.cli.views import sim_list_table, lab_list_table
from virl.helpers import find_virl, get_cml_client, get_cache_root


@click.command()
@click.option(
    "--all/--server",
    default=False,
    show_default=False,
    required=False,
    help="Display cached labs in addition to those on the server (default: server labs only)",
)
@click.option(
    "--all-users/--only-me",
    default=False,
    show_default=False,
    required=False,
    help="Display labs for all users (only if current user is an admin) (default: only show labs owned by me)",
)
def ls(all, all_users):
    """
    lists running labs and optionally those in the cache
    """
    server = VIRLServer()
    client = get_cml_client(server)
    labs = []
    cached_labs = None

    lab_ids = client.get_lab_list(all_users)
    for id in lab_ids:
        labs.append(client.join_existing_lab(id))

    if all:
        cached_labs = []
        cache_root = get_cache_root()
        if os.path.isdir(cache_root):
            for f in os.listdir(cache_root):
                lab_id = f
                cached_labs.append(CachedLab(lab_id, cache_root + "/" + f))

    try:
        pl = ViewerPlugin(viewer="lab")
        pl.visualize(labs=labs, cached_labs=cached_labs)
    except NoPluginError:
        lab_list_table(labs, cached_labs)


@click.command()
@click.option(
    "--all/--local",
    default=False,
    help=" \
Display all simulations or only ones from the current project (default)",
)
def ls1(all, **kwargs):
    """
    lists running simulations in the current project
    """

    server = VIRLServer()

    sim_dict = server.list_simulations()
    if not all:
        # only sims for this project
        dirpath = find_virl()
        foldername = os.path.basename(dirpath)
        for k in list(sim_dict):
            if not k.startswith(foldername):
                sim_dict.pop(k)

    sim_list_table(sim_dict)



import os
from . import BaseTest
from click.testing import CliRunner
from virl.cli.main import virl
from virl.swagger.app import app

try:
    from unittest.mock import patch
except ImportError:
    from mock import patch


class Tests(BaseTest):
    @patch("virl.cli.swagger.commands.subprocess.Popen", autospec=False)
    def test_virl_swagger_start(self, call_mock):
        runner = CliRunner()
        runner.invoke(virl, ["swagger", "start"])
        url = "http://localhost:5000"
        call_mock.assert_called_with(["open", url])

    def test_virl_swagger_status(self):
        with open("/tmp/virl_swagger.port", "w") as fh:
            fh.write("1234")
        with open("/tmp/virl_swagger.pid", "w") as fh:
            fh.write("1234")
        runner = CliRunner()
        result = runner.invoke(virl, ["swagger", "status"])
        expected_output = "VIRL swagger UI is running at http://localhost:1234"
        self.assertIn(expected_output, result.output)

    @patch("virl.cli.swagger.commands.subprocess.Popen", autospec=False)
    def test_virl_swagger_stop(self, call_mock):
        runner = CliRunner()
        result = runner.invoke(virl, ["swagger", "stop"])
        print(result.output)
        call_mock.assert_called_with("kill $(cat /tmp/virl_swagger.pid)", shell=True)


class SwaggerFlaskApp(BaseTest):
    def setUp(self):
        os.environ["VIRL_SWAGGER_HOST"] = "localhost"
        os.environ["VIRL_SWAGGER_USERNAME"] = "guest"
        os.environ["VIRL_SWAGGER_PASSWORD"] = "guest"
        os.environ["VIRL_SWAGGER_PORT"] = "1234"
        self.app = app
        app.testing = True
        self.app = app.test_client()

    def test_swagger_ui(self):
        resp = self.app.get("/")
        expected_data = b'ui.preauthorizeBasic("basicAuth", "guest", "guest")'
        self.assertEqual(resp.status_code, 200)
        self.assertIn(expected_data, resp.data)

    def test_swagger_spec(self):
        resp = self.app.get("/swagger.json")
        expected_data = b'"host": "localhost:1234",'
        self.assertEqual(resp.status_code, 200)
        self.assertIn(expected_data, resp.data)

    def tearDown(self):
        pass
        # os.close(self.db_fd)
        # os.unlink(flaskr.app.config['DATABASE'])



import click
import os
from virl.api import VIRLServer
from virl.helpers import get_cml_client, safe_join_existing_lab, get_current_lab, check_lab_cache, clear_current_lab


@click.command()
@click.option(
    "--force/--no-force",
    "-f",
    default=False,
    required=False,
    help="Stop and/or wipe a lab (if it's started) then remove it (default: False)",
)
@click.option(
    "--confirm/--no-confirm",
    show_default=False,
    default=True,
    help="Do not prompt for confirmation (default: prompt)",
    required=False,
)
@click.option(
    "--from-cache/--no-from-cache",
    default=False,
    required=False,
    show_default=False,
    help="Remove the lab from the cache (default: do not remove from cache)",
)
def rm(force, confirm, from_cache):
    """
    remove a lab
    """
    server = VIRLServer()
    client = get_cml_client(server)

    current_lab = get_current_lab()
    if current_lab:
        lab = safe_join_existing_lab(current_lab, client)
        if lab:
            if lab.is_active() and force:
                lab.stop(wait=True)

            if lab.state() != "DEFINED_ON_CORE" and force:
                lab.wipe(wait=True)

            # Check again just to be sure.
            if lab.state() == "DEFINED_ON_CORE":
                ret = "y"
                if confirm:
                    ret = input("Are you sure you want to remove lab {} (ID: {}) [y/N]? ".format(lab.title, current_lab))
                if ret.lower().startswith("y"):
                    # We need to save the lab's title before we remove it.
                    title = lab.title
                    lab.remove()
                    click.secho("Lab {} (ID: {}) removed".format(title, current_lab))
                    if from_cache:
                        try:
                            os.remove(check_lab_cache(current_lab))
                        except OSError:
                            # File doesn't exist.
                            pass

                        click.secho("Removed lab {} from cache".format(current_lab))
                    clear_current_lab()
                else:
                    click.secho("Not removing lab {} (ID: {})".format(lab.title, current_lab))

            else:
                click.secho(
                    "Lab {} (ID: {}) is either active or not wiped; either down and wipe it or use --force".format(lab.title, current_lab),
                    fg="red",
                )
                exit(1)
        else:
            click.secho("Unable to find lab {}".format(current_lab), fg="red")
            exit(1)
    else:
        click.secho("Current lab is not set", fg="red")
        exit(1)



from . import BaseCMLTest
from click.testing import CliRunner
import requests_mock

try:
    from unittest.mock import patch
except ImportError:
    from mock import patch


class CMLConsoleTests(BaseCMLTest):
    def test_cml_console_display(self):
        with requests_mock.Mocker() as m:
            # Mock the request to return what we expect from the API.
            self.setup_mocks(m)
            virl = self.get_virl()
            runner = CliRunner()
            result = runner.invoke(virl, ["console", "rtr-1", "--display"])
            self.assertEqual(0, result.exit_code)

    @patch("virl.cli.console.commands.call", autospec=False)
    def test_cml_console_connect(self, call_mock):
        with requests_mock.Mocker() as m:
            # Mock the request to return what we expect from the API.
            self.setup_mocks(m)
            virl = self.get_virl()
            runner = CliRunner()
            runner.invoke(virl, ["console", "rtr-1"])
            call_mock.assert_called_once_with(["ssh", "-t", "admin@localhost", "open", "/5f0d96/n1/0"])



from . import BaseCMLTest
from click.testing import CliRunner
import requests_mock
import os

try:
    from unittest.mock import patch
except ImportError:
    from mock import patch  # noqa


class CMLTestRm(BaseCMLTest):
    __m = None

    def setup_mocks(self, m):
        super().setup_mocks(m)
        self.__m = m
        m.delete(self.get_api_path("labs/{}".format(self.get_alt_id())), json="DELETED")

    def wipe_lab(self, req, ctx):
        self.__m.get(self.get_api_path("labs/{}/state".format(self.get_alt_id())), json="DEFINED_ON_CORE")
        self.__m.get(self.get_api_path("labs/{}/check_if_converged".format(self.get_alt_id())), json=True)
        return "WIPED"

    @patch("virl.cli.rm.commands.input", autospec=False, return_value="y")
    def test_cml_rm(self, input_mock):
        with requests_mock.Mocker() as m:
            # Mock the request to return what we expect from the API.
            self.setup_mocks(m)
            m.get(self.get_api_path("labs/{}/state".format(self.get_alt_id())), json="DEFINED_ON_CORE")
            virl = self.get_virl()
            runner = CliRunner()
            runner.invoke(virl, ["use", "--id", self.get_alt_id()])
            result = runner.invoke(virl, ["rm"])
            self.assertEqual(0, result.exit_code)
            self.assertNotIn("Not removing lab", result.output)

    @patch("virl.cli.rm.commands.input", autospec=False, return_value="y")
    def test_cml_rm_force(self, input_mock):
        with requests_mock.Mocker() as m:
            # Mock the request to return what we expect from the API.
            self.setup_mocks(m)
            m.put(self.get_api_path("labs/{}/wipe".format(self.get_alt_id())), json=self.wipe_lab)
            virl = self.get_virl()
            runner = CliRunner()
            runner.invoke(virl, ["use", "--id", self.get_alt_id()])
            result = runner.invoke(virl, ["rm", "--force"])
            self.assertEqual(0, result.exit_code)

    @patch("virl.cli.rm.commands.input", autospec=False, return_value="y")
    def test_cml_rm_no_confirm(self, input_mock):
        with requests_mock.Mocker() as m:
            # Mock the request to return what we expect from the API.
            self.setup_mocks(m)
            m.get(self.get_api_path("labs/{}/state".format(self.get_alt_id())), json="DEFINED_ON_CORE")
            virl = self.get_virl()
            runner = CliRunner()
            runner.invoke(virl, ["use", "--id", self.get_alt_id()])
            result = runner.invoke(virl, ["rm", "--no-confirm"])
            self.assertEqual(0, result.exit_code)
            input_mock.assert_not_called()

    @patch("virl.cli.rm.commands.input", autospec=False, return_value="y")
    def test_cml_rm_from_cache(self, input_mock):
        with requests_mock.Mocker() as m:
            # Mock the request to return what we expect from the API.
            self.setup_mocks(m)
            m.get(self.get_api_path("labs/{}/state".format(self.get_alt_id())), json="DEFINED_ON_CORE")
            virl = self.get_virl()
            runner = CliRunner()
            runner.invoke(virl, ["use", "--id", self.get_alt_id()])
            result = runner.invoke(virl, ["rm", "--from-cache"])
            self.assertEqual(0, result.exit_code)
            self.assertIn("Removed lab {} from cache".format(self.get_alt_id()), result.output)

    @patch("virl.cli.rm.commands.input", autospec=False, return_value="n")
    def test_cml_rm_denied(self, input_mock):
        with requests_mock.Mocker() as m:
            # Mock the request to return what we expect from the API.
            self.setup_mocks(m)
            m.get(self.get_api_path("labs/{}/state".format(self.get_alt_id())), json="DEFINED_ON_CORE")
            virl = self.get_virl()
            runner = CliRunner()
            runner.invoke(virl, ["use", "--id", self.get_alt_id()])
            result = runner.invoke(virl, ["rm"])
            self.assertEqual(0, result.exit_code)
            self.assertIn("Not removing lab", result.output)

    def test_cml_rm_not_wiped(self):
        with requests_mock.Mocker() as m:
            # Mock the request to return what we expect from the API.
            self.setup_mocks(m)
            virl = self.get_virl()
            runner = CliRunner()
            runner.invoke(virl, ["use", "--id", self.get_alt_id()])
            result = runner.invoke(virl, ["rm"])
            self.assertEqual(1, result.exit_code)
            self.assertIn("either active or not wiped", result.output)

    def test_cml_rm_no_lab(self):
        try:
            os.remove(".virl/current_cml_lab")
        except OSError:
            pass

        with requests_mock.Mocker() as m:
            # Mock the request to return what we expect from the API.
            self.setup_mocks(m)
            virl = self.get_virl()
            runner = CliRunner()
            result = runner.invoke(virl, ["rm"])
            self.assertEqual(1, result.exit_code)
            self.assertIn("Current lab is not set", result.output)

    def test_cml_rm_bogus_lab(self):
        try:
            os.remove(".virl/current_cml_lab")
        except OSError:
            pass

        src_dir = os.path.realpath(".virl")
        with open(".virl/cached_cml_labs/123456", "w") as fd:
            fd.write("lab: bogus\n")

        os.symlink("{}/cached_cml_labs/123456".format(src_dir), "{}/current_cml_lab".format(src_dir))

        with requests_mock.Mocker() as m:
            # Mock the request to return what we expect from the API.
            self.setup_mocks(m)
            virl = self.get_virl()
            runner = CliRunner()
            result = runner.invoke(virl, ["rm"])
            os.remove(".virl/cached_cml_labs/123456")
            os.remove(".virl/current_cml_lab")
            self.assertEqual(1, result.exit_code)
            self.assertIn("Unable to find lab 123456", result.output)



from . import BaseCMLTest
from click.testing import CliRunner
import requests_mock
import os


class CMLStartTests(BaseCMLTest):
    def test_cml_start(self):
        with requests_mock.Mocker() as m:
            # Mock the request to return what we expect from the API.
            m.put(self.get_api_path("labs/{}/nodes/n2/state/start".format(self.get_test_id())), json=None)
            self.setup_mocks(m)
            virl = self.get_virl()
            runner = CliRunner()
            result = runner.invoke(virl, ["start", "rtr-2"])
            self.assertEqual(0, result.exit_code)
            self.assertNotIn("Node rtr-2 is already active", result.output)

    def test_cml_start_already_active(self):
        with requests_mock.Mocker() as m:
            # Mock the request to return what we expect from the API.
            self.setup_mocks(m)
            virl = self.get_virl()
            runner = CliRunner()
            result = runner.invoke(virl, ["start", "rtr-1"])
            self.assertEqual(0, result.exit_code)
            self.assertIn("Node rtr-1 is already active", result.output)

    def test_cml_start_bogus_node(self):
        with requests_mock.Mocker() as m:
            # Mock the request to return what we expect from the API.
            self.setup_mocks(m)
            virl = self.get_virl()
            runner = CliRunner()
            result = runner.invoke(virl, ["start", "rtr-3"])
            self.assertEqual(1, result.exit_code)
            self.assertIn("Node rtr-3 was not found in lab {}".format(self.get_test_id()), result.output)

    def test_cml_start_bogus_lab(self):
        src_dir = os.path.realpath(".virl")
        try:
            os.remove(".virl/current_cml_lab")
        except OSError:
            pass

        with open(".virl/cached_cml_labs/123456", "w") as fd:
            fd.write("lab: bogus\n")

        os.symlink("{}/cached_cml_labs/123456".format(src_dir), "{}/current_cml_lab".format(src_dir))
        with requests_mock.Mocker() as m:
            # Mock the request to return what we expect from the API.
            self.setup_mocks(m)
            virl = self.get_virl()
            runner = CliRunner()
            result = runner.invoke(virl, ["start", "rtr-1"])
            os.remove(".virl/current_cml_lab")
            os.remove(".virl/cached_cml_labs/123456")
            self.assertEqual(1, result.exit_code)
            self.assertIn("Unable to find lab 123456", result.output)

    def test_cml_start_no_current_lab(self):
        try:
            os.remove(".virl/current_cml_lab")
        except OSError:
            pass

        with requests_mock.Mocker() as m:
            # Mock the request to return what we expect from the API.
            self.setup_mocks(m)
            virl = self.get_virl()
            runner = CliRunner()
            result = runner.invoke(virl, ["start", "rtr-1"])
            self.assertEqual(1, result.exit_code)
            self.assertIn("No current lab set", result.output)



import click
import os
from virl.api import VIRLServer

# from virl.swagger.app import app
import subprocess

PIDFILE = "/tmp/virl_swagger.pid"


@click.group()
def swagger1():
    """
    manage local swagger ui server
    """
    pass


@click.command()
@click.argument("port", default=5000)
def start1(port):
    """
    starts swagger ui for virl (mac only)
    """
    server = VIRLServer()
    os.environ["VIRL_SWAGGER_HOST"] = server.host
    os.environ["VIRL_SWAGGER_PORT"] = str(server.port)
    os.environ["VIRL_SWAGGER_USERNAME"] = server.user
    os.environ["VIRL_SWAGGER_PASSWORD"] = server.passwd
    os.environ["VIRL_SWAGGER_UI_PORT"] = str(port)

    with open("/tmp/virl_swagger.port", "w") as fh:
        fh.write(str(port))
    subprocess.Popen(["gunicorn", "-D", "virl.swagger.app:app", "-p", "/tmp/virl_swagger.pid", "-b", "127.0.0.1:{}".format(port)])

    subprocess.Popen(["open", "http://localhost:{}".format(port)])


@click.command()
@click.argument("port", default=5000)
def stop1(port):
    """
    stops swagger ui for virl (mac only)
    """
    subprocess.Popen("kill $(cat /tmp/virl_swagger.pid)", shell=True)
    portfile = "/tmp/virl_swagger.port"
    try:
        os.remove(portfile)
    except IOError:
        pass
    except OSError:
        pass


@click.command()
@click.argument("port", default=5000)
def status1(port):
    """
    show status of swagger ui for virl
    """
    pidfile = "/tmp/virl_swagger.pid"
    portfile = "/tmp/virl_swagger.port"

    if os.path.isfile(pidfile):
        with open(pidfile, "r") as fh:
            pid = fh.read()
    if os.path.isfile(portfile):
        with open(portfile, "r") as fh:
            port = fh.read()
        print("pid: {}".format(pid))
        print("VIRL swagger UI is running at http://localhost:{}".format(port))

    else:
        print("not running")


swagger1.add_command(start1, name="start")
swagger1.add_command(stop1, name="stop")
swagger1.add_command(status1, name="status")



from . import BaseTest
from click.testing import CliRunner
import requests_mock
from virl.cli.main import virl


class NodesTest(BaseTest):

    def mock_response(self):
        sim_response = {
            "guest|TEST_ENV|virl|router1": {
                "Status": "ACTIVE",
                "simLaunch": "2018-04-04T14:25:12.916689",
                "PortConsole": 17000,
                "NodeName": "router1",
                "simExpires": None,
                "managementIP": "1.1.1.1",
                "SerialPorts": 2,
                "SimulationHost": "5.5.5.5",
                "NodeSubtype": "IOSv",
                "simStatus": "ACTIVE",
                "Reachable": True,
                "PortMonitor": 17001,
                "managementProtocol": "telnet",
                "managementProxy": "jumphost",
                "VncConsole": False,
                "simID": "TEST_ENV",
                "Annotation": "REACHABLE"
            },
            "guest|TEST_ENV|virl|router2": {
                "Status": "BUILDING",
                "simLaunch": "2018-04-04T14:25:12.916689",
                "PortConsole": 17003,
                "NodeName": "router2",
                "simExpires": None,
                "managementIP": "2.2.2.2",
                "SerialPorts": 2,
                "SimulationHost": "5.5.5.5",
                "NodeSubtype": "IOSv",
                "simStatus": "ACTIVE",
                "Reachable": True,
                "PortMonitor": 17004,
                "managementProtocol": "telnet",
                "managementProxy": "jumphost",
                "VncConsole": False,
                "simID": "TEST_ENV",
                "Annotation": "REACHABLE"
            },
            "guest|TEST_ENV|virl|router3": {
                "Status": "",
                "simLaunch": "2018-04-04T14:25:12.916689",
                "PortConsole": 17003,
                "NodeName": "router2",
                "simExpires": None,
                "managementIP": "2.2.2.2",
                "SerialPorts": 2,
                "SimulationHost": "5.5.5.5",
                "NodeSubtype": "IOSv",
                "simStatus": "ACTIVE",
                "Reachable": True,
                "PortMonitor": 17004,
                "managementProtocol": "telnet",
                "managementProxy": "jumphost",
                "VncConsole": False,
                "simID": "TEST_ENV",
                "Annotation": "REACHABLE"
            },
            "guest|TEST_ENV|virl|mgmt-lxc": {
                "Status": "",
                "simLaunch": "2018-04-04T14:25:12.916689",
                "PortConsole": 17003,
                "NodeName": "router2",
                "simExpires": None,
                "managementIP": "2.2.2.2",
                "SerialPorts": 2,
                "SimulationHost": "5.5.5.5",
                "NodeSubtype": "LXC FLAT",
                "simStatus": "ACTIVE",
                "Reachable": True,
                "PortMonitor": 17004,
                "managementProtocol": "telnet",
                "managementProxy": "jumphost",
                "VncConsole": False,
                "simID": "TEST_ENV",
                "Annotation": "REACHABLE"
            }


        }
        return sim_response

    def test_virl_nodes(self):
        with requests_mock.mock() as m:
            # Mock the request to return what we expect from the API.
            m.get('http://localhost:19399/roster/rest/',
                  json=self.mock_response())
            runner = CliRunner()
            result = runner.invoke(virl, ["nodes"])
            self.assertEqual(0, result.exit_code)

    def test_virl_nodes_in_env(self):
        with requests_mock.mock() as m:
            # Mock the request to return what we expect from the API.
            m.get('http://localhost:19399/roster/rest/',
                  json=self.mock_response())
            runner = CliRunner()
            result = runner.invoke(virl, ["nodes", "foo"])
            self.assertEqual(0, result.exit_code)



import click
from virl.api import VIRLServer
from virl.helpers import get_cml_client


@click.command()
@click.argument("image", nargs=1)
@click.option("-f", "--filename", required=False, metavar="<filename>", help="filename to save to")
def export(image, filename):
    """
    export an image definition
    """

    server = VIRLServer()
    client = get_cml_client(server)

    if not filename:
        filename = image + ".yaml"

    defs = client.definitions

    try:
        idef = defs.download_image_definition(image)
    except Exception as e:
        click.secho("Failed to download image definition for {}: {}".format(image, e), fg="red")
        exit(1)
    else:
        with open(filename, "w") as fd:
            fd.write(idef)



from . import BaseCMLTest
from .mocks.github import MockGitHub  # noqa
from click.testing import CliRunner
import requests_mock
import os

try:
    from unittest.mock import patch
except ImportError:
    from mock import patch  # noqa


class TestCMLUp(BaseCMLTest):
    def get_up_id(self):
        return "7e5712"

    def setup_mocks(self, m):
        super().setup_mocks(m)
        m.post(self.get_api_path("import?title=Fake%20Lab"), json={"id": self.get_up_id()})
        m.get(self.get_api_path("labs/{}/topology?exclude_configurations=False".format(self.get_up_id())), json=TestCMLUp.get_fake_topology)
        m.get(self.get_api_path("labs/{}/state".format(self.get_up_id())), json="STOPPED")
        m.put(self.get_api_path("labs/{}/start".format(self.get_up_id())), json="STARTED")
        m.put(self.get_api_path("labs/{}/start".format(self.get_alt_id())), json="STARTED")
        m.get(self.get_api_path("labs/{}/download".format(self.get_up_id())), text=MockGitHub.get_topology)
        m.get(self.get_api_path("labs/{}/lab_element_state".format(self.get_up_id())), json=TestCMLUp.get_fake_element_state)

    @staticmethod
    def get_fake_element_state(req, ctx):
        response = {
            "nodes": {"n0": "BOOTED", "n1": "BOOTED"},
            "links": {"l0": "STARTED"},
            "interfaces": {"i0": "STARTED", "i1": "STARTED", "i2": "STARTED", "i3": "STARTED", "i4": "STARTED", "i5": "STARTED"},
        }
        return response

    @staticmethod
    def get_fake_topology(req, ctx):
        response = {
            "nodes": [
                {
                    "id": "n0",
                    "data": {
                        "node_definition": "nxosv",
                        "image_definition": "nxosv-7-3-0",
                        "label": "bt-test-sw",
                        "configuration": "hostname inserthostname_here",
                        "x": -350,
                        "y": 0,
                        "state": "STOPPED",
                        "ram": None,
                        "cpus": None,
                        "cpu_limit": None,
                        "data_volume": None,
                        "boot_disk_size": None,
                        "tags": [],
                    },
                },
                {
                    "id": "n1",
                    "data": {
                        "node_definition": "external_connector",
                        "image_definition": None,
                        "label": "ext-conn-0",
                        "configuration": "bridge0",
                        "x": 0,
                        "y": 0,
                        "state": "STOPPED",
                        "ram": None,
                        "cpus": None,
                        "cpu_limit": None,
                        "data_volume": None,
                        "boot_disk_size": None,
                        "tags": [],
                    },
                },
            ],
            "links": [{"id": "l0", "interface_a": "i1", "interface_b": "i5", "data": {"state": "DEFINED_ON_CORE"}}],
            "interfaces": [
                {"id": "i0", "node": "n0", "data": {"label": "Loopback0", "slot": None, "state": "STOPPED", "type": "loopback"}},
                {"id": "i1", "node": "n0", "data": {"label": "mgmt0", "slot": 0, "state": "STOPPED", "type": "physical"}},
                {"id": "i2", "node": "n0", "data": {"label": "Ethernet2/1", "slot": 1, "state": "STOPPED", "type": "physical"}},
                {"id": "i3", "node": "n0", "data": {"label": "Ethernet2/2", "slot": 2, "state": "STOPPED", "type": "physical"}},
                {"id": "i4", "node": "n0", "data": {"label": "Ethernet2/3", "slot": 3, "state": "STOPPED", "type": "physical"}},
                {"id": "i5", "node": "n1", "data": {"label": "port", "slot": 0, "state": "STOPPED", "type": "physical"}},
            ],
            "lab_notes": "",
            "lab_title": "Fake Lab",
            "lab_description": "",
            "state": "STOPPED",
            "created_timestamp": 1589294717.9075089,
            "cluster_id": "cluster_1",
            "version": "0.0.3",
        }
        return response

    def setUp(self):
        super().setUp()

        try:
            os.remove(".virl/current_cml_lab")
            os.system("cp -f tests/v2/static/fake_repo_topology.yaml topology.yaml")

        except OSError:
            pass

    def test_cml_up(self):
        with requests_mock.Mocker() as m:
            # Mock the request to return what we expect from the API.
            self.setup_mocks(m)
            virl = self.get_virl()
            runner = CliRunner()
            result = runner.invoke(virl, ["up"])
            self.assertEqual(0, result.exit_code)
            self.assertTrue(os.path.basename(os.readlink(".virl/current_cml_lab")) == self.get_up_id())
            self.assertTrue(os.path.isfile(os.readlink(".virl/current_cml_lab")))
            self.assertIn("Starting lab", result.output)

    def test_cml_up_by_name(self):
        with requests_mock.Mocker() as m:
            # Mock the request to return what we expect from the API.
            self.setup_mocks(m)
            virl = self.get_virl()
            runner = CliRunner()
            result = runner.invoke(virl, ["up", "--lab-name", self.get_alt_title()])
            self.assertEqual(0, result.exit_code)
            self.assertTrue(os.path.basename(os.readlink(".virl/current_cml_lab")) == self.get_alt_id())
            self.assertTrue(os.path.isfile(os.readlink(".virl/current_cml_lab")))

    def test_cml_up_by_id(self):
        with requests_mock.Mocker() as m:
            # Mock the request to return what we expect from the API.
            self.setup_mocks(m)
            virl = self.get_virl()
            runner = CliRunner()
            result = runner.invoke(virl, ["up", "--id", self.get_alt_id()])
            self.assertEqual(0, result.exit_code)
            self.assertTrue(os.path.basename(os.readlink(".virl/current_cml_lab")) == self.get_alt_id())
            self.assertTrue(os.path.isfile(os.readlink(".virl/current_cml_lab")))

    def test_cml_up_bad_yaml(self):
        try:
            os.system("cp tests/v2/static/fake_repo_bad_topology.yaml topology.yaml")
        except OSError:
            pass

        with requests_mock.Mocker() as m:
            # Mock the request to return what we expect from the API.
            self.setup_mocks(m)
            virl = self.get_virl()
            runner = CliRunner()
            result = runner.invoke(virl, ["up"])
            self.assertEqual(1, result.exit_code)
            self.assertIn("does not appear to be a YAML-formatted CML topology file", result.output)

    def test_cml_up_provision(self):
        with requests_mock.Mocker() as m:
            # Mock the request to return what we expect from the API.
            self.setup_mocks(m)
            virl = self.get_virl()
            runner = CliRunner()
            result = runner.invoke(virl, ["up", "--provision"])
            self.assertEqual(0, result.exit_code)
            self.assertIn("Waiting for all nodes to be online", result.output)

    def test_cml_up_no_start(self):
        with requests_mock.Mocker() as m:
            # Mock the request to return what we expect from the API.
            self.setup_mocks(m)
            virl = self.get_virl()
            runner = CliRunner()
            result = runner.invoke(virl, ["up", "--no-start"])
            self.assertEqual(0, result.exit_code)
            self.assertNotIn("Starting lab", result.output)

    def test_cml_up_after_use(self):
        super().setUp()
        with requests_mock.Mocker() as m:
            # Mock the request to return what we expect from the API.
            self.setup_mocks(m)
            virl = self.get_virl()
            runner = CliRunner()
            result = runner.invoke(virl, ["up"])
            self.assertEqual(0, result.exit_code)
            self.assertIn(
                "Lab {} (ID: {}) is already set as the current lab".format(self.get_test_title(), self.get_test_id()), result.output
            )

    def test_cml_up_running_lab(self):
        with requests_mock.Mocker() as m:
            # Mock the request to return what we expect from the API.
            self.setup_mocks(m)
            virl = self.get_virl()
            runner = CliRunner()
            result = runner.invoke(virl, ["up", "--id", self.get_test_id()])
            self.assertEqual(0, result.exit_code)
            self.assertIn("Lab is already running (ID: {}, Title: {})".format(self.get_test_id(), self.get_test_title()), result.output)

    @patch("virl.cli.up.commands.call", autospec=False)
    def test_cml_up_no_lab(self, call_mock):
        try:
            os.remove("topology.yaml")
        except OSError:
            pass

        with requests_mock.Mocker() as m:
            # Mock the request to return what we expect from the API.
            self.setup_mocks(m)
            virl = self.get_virl()
            runner = CliRunner()
            result = runner.invoke(virl, ["up"])
            self.assertEqual(1, result.exit_code)
            self.assertIn("Could not find a lab to start", result.output)

    def test_cml_up_bogus_current_lab(self):
        src_dir = os.path.realpath(".virl")
        with open(".virl/cached_cml_labs/123456", "w") as fd:
            fd.write("lab: bogus\n")

        os.symlink("{}/cached_cml_labs/123456".format(src_dir), "{}/current_cml_lab".format(src_dir))
        with requests_mock.Mocker() as m:
            # Mock the request to return what we expect from the API.
            self.setup_mocks(m)
            virl = self.get_virl()
            runner = CliRunner()
            result = runner.invoke(virl, ["up"])
            os.remove(".virl/cached_cml_labs/123456")
            self.assertIn("Current lab is already set to {}, but that lab is not on server".format("123456"), result.output)

    @patch("virl.cli.up.commands.call", autospec=False, return_value=0)
    def test_cml_up_from_repo(self, call_mock):
        try:
            os.remove("topology.yaml")
        except OSError:
            pass

        try:
            os.remove("topology.virl")
        except OSError:
            pass

        with requests_mock.Mocker() as m:
            self.setup_mocks(m)
            topo_url = "https://raw.githubusercontent.com/"
            topo_url += "foo/bar/master/topology.yaml"
            m.get(topo_url, json=MockGitHub.get_topology)
            virl = self.get_virl()
            runner = CliRunner()
            runner.invoke(virl, ["up", "foo/bar"])
            # The "call" function was called twice
            self.assertEqual(2, len(call_mock.mock_calls))



from . import BaseTest
from click.testing import CliRunner
import requests_mock
from virl.cli.main import virl
from .mocks import MockVIRLServer
# import filecmp


class Tests(BaseTest):

    def test_virl_generate_ansible_yaml(self):
        with requests_mock.mock() as m:
            # Mock the request to return what we expect from the API.
            roster_url = 'http://localhost:19399/roster/rest/'
            m.get(roster_url, json=MockVIRLServer.get_sim_roster())
            export_url = 'http://localhost:19399/simengine/rest/export/'
            export_url += 'TEST_ENV?running-configs=config&updated=true'
            m.get(export_url, text=MockVIRLServer.export())
            interface_url = 'http://localhost:19399/simengine/rest/interfaces/'
            interface_url += 'TEST_ENV'
            m.get(interface_url, json=MockVIRLServer.get_interfaces())
            runner = CliRunner()
            result = runner.invoke(virl, ["generate", "ansible"])
            # same = filecmp.cmp('default_inventory.yaml',
            #                    'tests/static/ansible_yaml_inventory')
            # self.assertTrue(same)
            self.assertEqual(0, result.exit_code)

    def test_virl_generate_ansible_ini(self):
        with requests_mock.mock() as m:
            # Mock the request to return what we expect from the API.
            roster_url = 'http://localhost:19399/roster/rest/'
            m.get(roster_url, json=MockVIRLServer.get_sim_roster())
            export_url = 'http://localhost:19399/simengine/rest/export/'
            export_url += 'TEST_ENV?running-configs=config&updated=true'
            m.get(export_url, text=MockVIRLServer.export())
            interface_url = 'http://localhost:19399/simengine/rest/interfaces/'
            interface_url += 'TEST_ENV'
            m.get(interface_url, json=MockVIRLServer.get_interfaces())
            runner = CliRunner()
            result = runner.invoke(virl, ["generate", "ansible",
                                          "--style", "ini"])
            # same = filecmp.cmp('default_inventory.ini',
            #                    'tests/static/ansible_ini_inventory')
            # self.assertTrue(same)
            self.assertEqual(0, result.exit_code)



from . import BaseTest
from .mocks import MockVIRLServer
from .mocks.nso import MockNSOServer
from click.testing import CliRunner
import requests_mock
from virl.cli.main import virl


class Tests(BaseTest):
    def test_virl_generate_nso(self):
        with requests_mock.mock() as m:
            # Mock the request to return what we expect from the API.
            roster_url = "http://localhost:19399/roster/rest/"
            m.get(roster_url, json=MockVIRLServer.get_sim_roster())
            export_url = "http://localhost:19399/simengine/rest/export/"
            export_url += "TEST_ENV?running-configs=config&updated=true"
            m.get(export_url, text=MockVIRLServer.export())
            interface_url = "http://localhost:19399/simengine/rest/interfaces/"
            interface_url += "TEST_ENV"
            m.get(interface_url, json=MockVIRLServer.get_interfaces())

            # mock the responses we expect from NSO
            devices_url = "http://localhost:8080/api/config/devices/"
            m.patch(devices_url, json=MockNSOServer.update_devices())
            sync_url = "http://localhost:8080/api/running/devices/"
            sync_url += "_operations/sync-from"
            m.post(sync_url, json=MockNSOServer.perform_sync_from())
            ned_url = "http://localhost:8080/api/config/devices/ned-ids/ned-id"
            m.get(ned_url, json=MockNSOServer.get_ned_list())
            module_url = "http://localhost:8080/api/config/modules-state/module"
            m.get(module_url, json=MockNSOServer.get_module_list())
            runner = CliRunner()
            result = runner.invoke(virl, ["generate", "nso", "--syncfrom"])
            self.assertEqual(0, result.exit_code)



import click
from virl.api import VIRLServer
from virl.cli.views import flavor_list_table


@click.command()
@click.option('--flavor', default=None)
def ls(**kwargs):
    """
    list all flavors or the details of a specific flavor
    """

    flavor = kwargs.get('flavor')
    server = VIRLServer(port=80)

    # Regardless of the argument, we have to get all the flavors
    # In the case of no arg, we print them all.
    # In the case of an arg, we have to go back and get details.
    r = server.get_flavors()

    if flavor:
        for f in list(r):
            if f['name'] == flavor:
                flavor_list_table([f])
                break
    else:
        flavor_list_table(r)



import requests


def get_repos(org='virlfiles', query=None):
    resp = requests.get('https://api.github.com/orgs/{}/repos'.format(org))
    ret = list()

    if query is None:
        return resp.json()

    for repo in resp.json():
        name = repo['name']
        descr = repo['description']
        if query in name:
            ret.append(repo)
        if descr and query in descr:
            ret.append(repo)

    return ret



from . import BaseCMLTest
from click.testing import CliRunner
import requests_mock
import textwrap
import os
import traceback


class CMLDefinitionsTest(BaseCMLTest):
    def test_cml_image_definitions_import(self):
        with requests_mock.Mocker() as m:
            # Mock the request to return what we expect from the API.
            self.setup_mocks(m)
            m.post(self.get_api_path("image_definitions/"), json=True)  # virl2_client 2.2.1 uses URI that ends in slashes
            virl = self.get_virl()
            runner = CliRunner()
            result = runner.invoke(
                virl,
                [
                    "definitions",
                    "images",
                    "import",
                    "definition",
                    "-f",
                    os.path.join(os.path.dirname(__file__), "static/fake_image_definitions.yaml"),
                ],
            )
            self.assertEqual(0, result.exit_code, result.stdout)

    def test_node_definitions_list(self):
        with requests_mock.Mocker() as m:
            with open(os.path.join(os.path.dirname(__file__), "static/response_get_node_defs.json"), "rb") as fh_node_defs:
                # Mock the request to return what we expect from the API.
                self.setup_mocks(m)
                m.get(
                    self.get_api_path("node_definitions/"), body=fh_node_defs, headers={"content-type": "application/json; charset=utf-8"}
                )
                virl = self.get_virl()
                runner = CliRunner()
                result = runner.invoke(virl, ["definitions", "nodes", "ls"])
                if result.exception:
                    traceback.print_exception(*result.exc_info)
                    raise result.exception
                with open(os.path.join(os.path.dirname(__file__), "static/node_defs_list_output.txt")) as fh_node_defs:
                    output_text = fh_node_defs.read()
                    self.assertEquals(output_text, result.output)

    def test_node_definitions_list_one(self):
        with requests_mock.Mocker() as m:
            with open(os.path.join(os.path.dirname(__file__), "static/response_get_node_defs.json"), "rb") as fh_node_defs:
                # Mock the request to return what we expect from the API.
                self.setup_mocks(m)
                m.get(
                    self.get_api_path("node_definitions/"), body=fh_node_defs, headers={"content-type": "application/json; charset=utf-8"}
                )
                virl = self.get_virl()
                runner = CliRunner()
                result = runner.invoke(virl, ["definitions", "nodes", "ls", "--node", "nxosv9000"])
                if result.exception:
                    traceback.print_exception(*result.exc_info)
                    raise result.exception
                expected = textwrap.dedent(
                    """\
                    ╒═══════════╤════════════╤══════════════════════════╤══════════════════════╤════════╤════════╤══════════════════╕
                    │ ID        │ Label      │ Description              │   Max No. Interfaces │ RAM    │   CPUs │ Boot Disk Size   │
                    ╞═══════════╪════════════╪══════════════════════════╪══════════════════════╪════════╪════════╪══════════════════╡
                    │ nxosv9000 │ NX-OS 9000 │ Cisco Nexus 9000v Switch │                   65 │ 8.0 GB │      2 │ N/A              │
                    ╘═══════════╧════════════╧══════════════════════════╧══════════════════════╧════════╧════════╧══════════════════╛
                    """
                )
                self.assertEquals(expected, result.output)

    def test_node_definitions_list_legacy(self):
        """
        Check that we can still handle data in the legacy format that was
        returned by the API before CML 2.3.
        """
        with requests_mock.Mocker() as m:
            with open(os.path.join(os.path.dirname(__file__), "static/response_get_node_defs_cml22.json"), "rb") as fh_node_defs:
                # Mock the request to return what we expect from the API.
                self.setup_mocks(m)
                m.get(
                    self.get_api_path("node_definitions/"), body=fh_node_defs, headers={"content-type": "application/json; charset=utf-8"}
                )
                virl = self.get_virl()
                runner = CliRunner()
                result = runner.invoke(virl, ["definitions", "nodes", "ls"])
                if result.exception:
                    traceback.print_exception(*result.exc_info)
                    raise result.exception
                with open(os.path.join(os.path.dirname(__file__), "static/node_defs_list_output_cml22.txt")) as fh_node_defs:
                    output_text = fh_node_defs.read()
                    self.assertEquals(output_text, result.output)



import click
from virl.api import VIRLServer, ViewerPlugin, NoPluginError
from virl.cli.views import node_def_list_table
from virl.helpers import get_cml_client


@click.command()
@click.option("--node", default=None)
def ls(**kwargs):
    """
    list all node definitions or the details of a specific node definition
    """

    node = kwargs.get("node")
    server = VIRLServer()
    client = get_cml_client(server)
    pl = None

    # Regardless of the argument, we have to get all the node definitions
    # In the case of no arg, we print them all.
    # In the case of an arg, we have to go back and get details.
    defs_orig = client.definitions.node_definitions()

    # Create a new list of the *flattened* node definitions.  CML 2.3 removed the
    # extra "data" layer of nesting in the node def JSON format.  To make the rest
    # of the code work no matter which version of CML we're talking to, create a
    # list of node definitions, removing the extra layer of nesting if needed.
    defs = [f["data"] if "data" in f else f for f in defs_orig]

    try:
        pl = ViewerPlugin(viewer="node_def")
    except NoPluginError:
        pass

    if node:
        for f in list(defs):
            if f["id"] == node:
                if pl:
                    pl.visualize(node_defs=[f])
                else:
                    node_def_list_table([f])
                break
    else:
        if pl:
            pl.visualize(node_defs=defs)
        else:
            node_def_list_table(defs)



import os
import click
from virl.api import VIRLServer, CachedLab
from virl.helpers import get_current_lab, get_cml_client, safe_join_existing_lab, get_current_lab_link


@click.command()
def lid():
    """
    get the current lab title and ID
    """
    server = VIRLServer()
    client = get_cml_client(server)
    current_lab = get_current_lab()
    if current_lab:
        lab = safe_join_existing_lab(current_lab, client)
        # The lab really should be on the server.
        if not lab:
            try:
                lab = CachedLab(current_lab, get_current_lab_link())
            except Exception:
                pass

        if lab:
            click.echo("{} (ID: {})".format(lab.title, current_lab))
        else:
            click.secho("Current lab is set to {}, but is not on server or in cache!".format(current_lab), fg="red")


@click.command()
def sid():
    """
    gets sim id for local environment
    """

    server = VIRLServer()

    sim_dict = server.list_simulations()
    dirpath = os.getcwd()
    foldername = os.path.basename(dirpath)
    for k in list(sim_dict):
        if not k.startswith(foldername):
            sim_dict.pop(k)
    # can only accurately determine sim id if there is
    # only one sim running with our project name
    if len(sim_dict) == 1:
        click.echo(list(sim_dict)[0])



from .sims.sim_views import sim_list_table  # noqa
from .nodes.node_views import node_list_table1, node_list_table  # noqa
from .logs.logs_views import log_table  # noqa
from .generate.nso import sync_table  # noqa
from .flavors.flavor_views import flavor_list_table  # noqa
from .labs.lab_views import lab_list_table  # noqa
from .images.image_views import image_list_table  # noqa
from .node_defs.node_def_views import node_def_list_table  # noqa
from .license.license_views import license_details_table, license_features_table  # noqa



import click
from virl.cli.flavors.ls.commands import ls
from virl.cli.flavors.add.commands import add
from virl.cli.flavors.delete.commands import delete
from virl.cli.flavors.update.commands import update


@click.group()
def flavors1():
    """
    Manage VIRL Flavors Attributes
    """
    pass


flavors1.add_command(ls)
flavors1.add_command(add)
flavors1.add_command(delete)
flavors1.add_command(update)



import click
from virl.api import VIRLServer
from virl.helpers import get_cml_client, safe_join_existing_lab, get_current_lab, extract_configurations, cache_lab


@click.command()
@click.option("--update-cache/--no-update-cache", default=True, help="update the local cache (default: True)")
def extract(update_cache, **kwargs):
    """
    extract configurations from all nodes in a lab
    """
    server = VIRLServer()
    client = get_cml_client(server)

    current_lab = get_current_lab()
    if current_lab:
        lab = safe_join_existing_lab(current_lab, client)
        if lab:
            extract_configurations(lab)

            if update_cache:
                cache_lab(lab, force=True)
        else:
            click.secho("Failed to find running lab {}".format(current_lab), fg="red")
            exit(1)
    else:
        click.secho("Current lab is not set", fg="red")
        exit(1)



import click
from subprocess import call
from virl.api import VIRLServer, CachedLab
from virl.helpers import (
    generate_sim_id,
    check_sim_running,
    store_sim_info,
    get_cml_client,
    safe_join_existing_lab,
    safe_join_existing_lab_by_title,
    check_lab_cache,
    cache_lab,
    set_current_lab,
    get_current_lab,
    clear_current_lab,
    get_command,
)
import os
import time


def get_lab_title(fname):
    """
    Try and obtain a sensible title for the lab based on the filename
    and/or its contents.
    """
    # We need to do this to preserve any .virl extension to to tell CML this
    # is an older file.
    title = os.path.basename(fname)
    if not fname.lower().endswith(".virl"):
        title = os.path.splitext(fname)[0]
        # Load the lab YAML to try and extract its title property
        try:
            lab_stub = CachedLab("bogusid", fname)
        except Exception:
            # Someone may be trying to load a 1.x file without the .virl extension.
            click.secho(
                "File {} does not appear to be a YAML-formatted CML topology file."
                "If this is a CML/VIRL 1.x file, it must end with '.virl'".format(fname),
                fg="red",
            )
            exit(1)
        else:
            title = lab_stub.title

    return title


def start_lab(lab, provision=False):
    click.secho("Starting lab {} (ID: {})".format(lab.title, lab.id))
    lab.wait_for_convergence = False
    lab.start()
    cache_lab(lab)
    set_current_lab(lab.id)
    if provision:
        # Technically we need to block until all nodes are "reachable".
        # In the CML 2+ case, this means BOOTED.
        click.secho("Waiting for all nodes to be online...")
        ready = False
        while not ready:
            for n in lab.nodes():
                if not n.is_booted():
                    ready = False
                    break
                ready = True
            time.sleep(1)


@click.command()
@click.argument("repo", default="default")
@click.option(
    "-f",
    help="Lab file to launch, defaults to topology.yaml (or topology.virl if topology.yaml is not found)",
    required=False,
)
@click.option(
    "--provision/--noprovision",
    show_default=False,
    default=False,
    help="Blocks execution until all nodes are reachable (default: do not wait)",
    required=False,
)
@click.option(
    "--start/--no-start",
    show_default=False,
    default=True,
    help="Whether or not to start the lab after importing (default: start the lab)",
    required=False,
)
@click.option("--id", required=False, help="An existing lab ID to start (topology file is ignored, lab-name is ignored)")
@click.option("--lab-name", "-n", "--sim-name", required=False, help="An existing lab name to start (topology file is ignored)")
def up(repo=None, provision=False, start=True, **kwargs):
    """
    start a lab
    """
    def_fname = kwargs["f"]
    alt_fname = "topology.virl"
    fname = def_fname
    lid = kwargs["id"]
    lab_name = kwargs["lab_name"]
    lab = None
    clab = None

    server = VIRLServer()
    client = get_cml_client(server)

    current_lab = get_current_lab()
    if current_lab:
        clab = safe_join_existing_lab(current_lab, client)
        if not clab:
            click.secho("Current lab is already set to {}, but that lab is not on server; clearing it.".format(current_lab), fg="yellow")
            clear_current_lab()

    if not clab or fname or lid or lab_name:
        if clab:
            click.secho("WARNING: Current lab is set to {} (ID: {}); clearing it".format(clab.title, current_lab), fg="yellow")
            clear_current_lab()

        if not def_fname:
            def_fname = "topology.yaml"
            fname = def_fname

        if not os.path.isfile(def_fname) and os.path.isfile(alt_fname):
            fname = alt_fname

        if lid:
            lab = safe_join_existing_lab(lid, client)
            if not lab:
                # Check the cache
                existing = check_lab_cache(lid)
                if existing:
                    fname = existing

        if not lab and lab_name:
            lab = safe_join_existing_lab_by_title(lab_name, client)

        if not lab and os.path.isfile(fname):
            lname = get_lab_title(fname)
            click.secho("Importing lab {} from file {}".format(lname, fname))
            lab = client.import_lab_from_path(fname, title=lname)
        elif not lab:
            # try to pull from virlfiles
            if repo and (os.path.basename(fname) == "topology.yaml" or os.path.basename(fname) == "topology.virl"):
                rc = call([get_command(), "pull", repo])
                if rc == 0:
                    exit(call([get_command(), "up"]))

        if lab:
            if lab.is_active():
                cache_lab(lab)
                set_current_lab(lab.id)
                click.secho("Lab is already running (ID: {}, Title: {})".format(lab.id, lab.title))
            elif start:
                start_lab(lab, provision)

        else:
            click.secho("Could not find a lab to start.  Maybe try -f", fg="red")
            exit(1)
    elif clab:
        click.secho("Lab {} (ID: {}) is already set as the current lab".format(clab.title, current_lab))
        if not clab.is_active() and start:
            start_lab(clab, provision)


@click.command()
@click.argument("repo", default="default")
@click.option("-e", default="default", help="environment name", required=False)
@click.option(
    "-f",
    default="topology.virl",
    help=" \
VIRL file to launch, defaults to topology.virl",
    required=False,
)
@click.option(
    "--provision/--noprovision",
    show_default=False,
    default=False,
    help=" \
Blocks execution until all nodes are reachable.",
    required=False,
)
@click.option("--wait-time", default=10, help="max time (in minutes) to wait for nodes to come online", show_default=True)
def up1(repo=None, provision=False, **kwargs):
    """
    start a virl simulation
    """
    fname = kwargs["f"]
    env = kwargs["e"]
    wait_time = kwargs["wait_time"]

    if os.path.exists(fname):
        running = check_sim_running(env)
        if not running:
            click.secho("Creating {} environment from {}".format(env, fname))
            with open(fname) as fh:
                data = fh.read()
            server = VIRLServer()

            # we can expose fairly aribtary substitutions here...
            # anything that may differ usually related to networking....
            # <dirty hack>
            subs = {
                "{{ gateway }}": server.get_gateway_for_network("flat"),
                "{{ flat1_gateway }}": server.get_gateway_for_network("flat1"),
                "{{ dns_server }}": server.get_dns_server_for_network("flat"),
            }

            # also can change some VIRL/ANK defaults
            subs["rsa modulus 768"] = "rsa modulus 1024"

            for tag, value in subs.items():
                if tag in data:
                    if value:
                        # split off the braces
                        humanize = tag
                        click.secho("Localizing {} with: {}".format(humanize, value))
                        data = data.replace(tag, value)

            # </dirty hack>

            dirpath = os.getcwd()
            foldername = os.path.basename(dirpath)
            sim_name = "{}_{}_{}".format(foldername, env, generate_sim_id())
            resp = server.launch_simulation(sim_name, data)
            store_sim_info(resp.text, env=env)  # 'topology-2lkx2'

            if provision:
                nodes = server.get_node_list(sim_name)
                msg = "Waiting {} minutes for nodes to come online...."
                msg = msg.format(wait_time)
                click.secho(msg)
                maxtime = time.time() + 60 * int(wait_time)
                with click.progressbar(nodes) as all_nodes:
                    for node in all_nodes:
                        if time.time() > maxtime:
                            click.secho("")
                            click.secho("Max time expired", fg="red")
                            click.secho("All nodes may not be online", fg="red")
                            break
                        node_online = False
                        while not node_online:
                            if time.time() > maxtime:
                                break
                            time.sleep(20)
                            node_online = server.check_node_reachable(sim_name, node)
        else:
            click.secho("Sim {} already running".format(running))
    else:
        # try to pull from virlfiles
        if repo:
            call([get_command(), "pull", repo])
            call([get_command(), "up"])
        else:
            click.secho("Could not find topology.virl. Maybe try -f", fg="red")



import click
import os
from virl.api import VIRLServer
from virl.helpers import get_cml_client


@click.command()
@click.option("-f", "--filename", required=True, metavar="<filename>", help="path to local image file")
@click.option("--rename", required=False, metavar="<filename>", help="optional new name to give the file on the server")
def image_file(filename, rename):
    """
    import an image file
    """

    server = VIRLServer()
    client = get_cml_client(server)

    if not os.path.isfile(filename):
        click.secho("Image file {} does not exist or is not a file", fg="red")
        exit(1)
    else:
        defs = client.definitions
        try:
            defs.upload_image_file(filename, rename)
        except Exception as e:
            click.secho("Failed to import image file {}: {}".format(filename, e), fg="red")
            exit(1)



from . import BaseTest
from click.testing import CliRunner
from virl.cli.main import virl

try:
    from unittest.mock import patch
except ImportError:
    from mock import patch


class Tests(BaseTest):
    @patch("virl.cli.uwm.commands.subprocess.Popen", autospec=False)
    def test_virl_uwm(self, call_mock):
        runner = CliRunner()
        runner.invoke(virl, ["uwm"])
        url = "http://guest:guest@localhost/simulation/guest/TEST_ENV"
        call_mock.assert_called_with(["open", url])



from . import BaseCMLTest
from .mocks import MockCMLServer
from click.testing import CliRunner
import requests_mock
import os

try:
    from unittest.mock import patch
except ImportError:
    from mock import patch  # noqa


class CMLTestWipe(BaseCMLTest):
    __m = None

    def setup_mocks(self, m):
        super().setup_mocks(m)
        self.__m = m

    def stop_lab(self, req, ctx):
        self.__m.get(self.get_api_path("labs/{}/state".format(self.get_test_id())), json="DEFINED_ON_CORE")
        return "STOPPED"

    def wipe_lab(self, req, ctx):
        self.__m.get(self.get_api_path("labs/{}/check_if_converged".format(self.get_test_id())), json=True)
        return "WIPED"

    def stop_node(self, req, ctx):
        self.__m.get(
            self.get_api_path("labs/{}/lab_element_state".format(self.get_test_id())), json=MockCMLServer.get_lab_element_state_down
        )

    @patch("virl.cli.wipe.lab.commands.input", autospec=False, return_value="y")
    def test_cml_wipe_lab(self, input_mock):
        with requests_mock.Mocker() as m:
            # Mock the request to return what we expect from the API.
            self.setup_mocks(m)
            m.get(self.get_api_path("labs/{}/state".format(self.get_test_id())), json="DEFINED_ON_CORE")
            m.put(self.get_api_path("labs/{}/wipe".format(self.get_test_id())), json=self.wipe_lab)
            virl = self.get_virl()
            runner = CliRunner()
            result = runner.invoke(virl, ["wipe", "lab"])
            self.assertEqual(0, result.exit_code)
            self.assertNotIn("Not wiping lab", result.output)

    @patch("virl.cli.wipe.lab.commands.input", autospec=False, return_value="y")
    def test_cml_wipe_lab_force(self, input_mock):
        with requests_mock.Mocker() as m:
            # Mock the request to return what we expect from the API.
            self.setup_mocks(m)
            m.put(self.get_api_path("labs/{}/wipe".format(self.get_test_id())), json=self.wipe_lab)
            m.put(self.get_api_path("labs/{}/stop".format(self.get_test_id())), json=self.stop_lab)
            virl = self.get_virl()
            runner = CliRunner()
            result = runner.invoke(virl, ["wipe", "lab", "--force"])
            self.assertEqual(0, result.exit_code)

    @patch("virl.cli.wipe.lab.commands.input", autospec=False, return_value="y")
    def test_cml_wipe_lab_no_confirm(self, input_mock):
        with requests_mock.Mocker() as m:
            # Mock the request to return what we expect from the API.
            self.setup_mocks(m)
            m.get(self.get_api_path("labs/{}/state".format(self.get_test_id())), json="DEFINED_ON_CORE")
            m.put(self.get_api_path("labs/{}/wipe".format(self.get_test_id())), json=self.wipe_lab)
            virl = self.get_virl()
            runner = CliRunner()
            result = runner.invoke(virl, ["wipe", "lab", "--no-confirm"])
            self.assertEqual(0, result.exit_code)
            input_mock.assert_not_called()

    @patch("virl.cli.wipe.lab.commands.input", autospec=False, return_value="n")
    def test_cml_wipe_lab_denied(self, input_mock):
        with requests_mock.Mocker() as m:
            # Mock the request to return what we expect from the API.
            self.setup_mocks(m)
            m.get(self.get_api_path("labs/{}/state".format(self.get_test_id())), json="DEFINED_ON_CORE")
            m.put(self.get_api_path("labs/{}/wipe".format(self.get_test_id())), json=self.wipe_lab)
            virl = self.get_virl()
            runner = CliRunner()
            result = runner.invoke(virl, ["wipe", "lab"])
            self.assertEqual(0, result.exit_code)
            self.assertIn("Not wiping lab", result.output)

    def test_cml_wipe_lab_not_stopped(self):
        with requests_mock.Mocker() as m:
            # Mock the request to return what we expect from the API.
            self.setup_mocks(m)
            m.put(self.get_api_path("labs/{}/wipe".format(self.get_test_id())), json=self.wipe_lab)
            virl = self.get_virl()
            runner = CliRunner()
            result = runner.invoke(virl, ["wipe", "lab"])
            self.assertEqual(1, result.exit_code)
            self.assertIn("is active", result.output)

    def test_cml_wipe_lab_no_lab(self):
        try:
            os.remove(".virl/current_cml_lab")
        except OSError:
            pass

        with requests_mock.Mocker() as m:
            # Mock the request to return what we expect from the API.
            self.setup_mocks(m)
            virl = self.get_virl()
            runner = CliRunner()
            result = runner.invoke(virl, ["wipe", "lab"])
            self.assertEqual(1, result.exit_code)
            self.assertIn("Current lab is not set", result.output)

    def test_cml_wipe_lab_bogus_lab(self):
        try:
            os.remove(".virl/current_cml_lab")
        except OSError:
            pass

        src_dir = os.path.realpath(".virl")
        with open(".virl/cached_cml_labs/123456", "w") as fd:
            fd.write("lab: bogus\n")

        os.symlink("{}/cached_cml_labs/123456".format(src_dir), "{}/current_cml_lab".format(src_dir))

        with requests_mock.Mocker() as m:
            # Mock the request to return what we expect from the API.
            self.setup_mocks(m)
            virl = self.get_virl()
            runner = CliRunner()
            result = runner.invoke(virl, ["wipe", "lab"])
            os.remove(".virl/cached_cml_labs/123456")
            os.remove(".virl/current_cml_lab")
            self.assertEqual(1, result.exit_code)
            self.assertIn("Unable to find lab 123456", result.output)

    @patch("virl.cli.wipe.node.commands.input", autospec=False, return_value="y")
    def test_cml_wipe_node(self, input_mock):
        with requests_mock.Mocker() as m:
            # Mock the request to return what we expect from the API.
            self.setup_mocks(m)
            m.put(self.get_api_path("labs/{}/nodes/n2/wipe_disks".format(self.get_test_id())), json=True)
            virl = self.get_virl()
            runner = CliRunner()
            result = runner.invoke(virl, ["wipe", "node", "rtr-2"])
            self.assertEqual(0, result.exit_code)
            self.assertNotIn("Not wiping node", result.output)

    @patch("virl.cli.wipe.node.commands.input", autospec=False, return_value="y")
    def test_cml_wipe_node_force(self, input_mock):
        with requests_mock.Mocker() as m:
            # Mock the request to return what we expect from the API.
            self.setup_mocks(m)
            m.put(self.get_api_path("labs/{}/nodes/n1/state/stop".format(self.get_test_id())), json=self.stop_node)
            m.put(self.get_api_path("labs/{}/nodes/n1/wipe_disks".format(self.get_test_id())), json=True)
            virl = self.get_virl()
            runner = CliRunner()
            result = runner.invoke(virl, ["wipe", "node", "rtr-1", "--force"], catch_exceptions=False)
            self.assertEqual(0, result.exit_code)

    @patch("virl.cli.wipe.node.commands.input", autospec=False, return_value="y")
    def test_cml_wipe_node_no_confirm(self, input_mock):
        with requests_mock.Mocker() as m:
            # Mock the request to return what we expect from the API.
            self.setup_mocks(m)
            m.put(self.get_api_path("labs/{}/nodes/n2/wipe_disks".format(self.get_test_id())), json=True)
            virl = self.get_virl()
            runner = CliRunner()
            result = runner.invoke(virl, ["wipe", "node", "rtr-2", "--no-confirm"])
            self.assertEqual(0, result.exit_code)
            input_mock.assert_not_called()

    @patch("virl.cli.wipe.node.commands.input", autospec=False, return_value="n")
    def test_cml_wipe_node_denied(self, input_mock):
        with requests_mock.Mocker() as m:
            # Mock the request to return what we expect from the API.
            self.setup_mocks(m)
            virl = self.get_virl()
            runner = CliRunner()
            result = runner.invoke(virl, ["wipe", "node", "rtr-2"])
            self.assertEqual(0, result.exit_code)
            self.assertIn("Not wiping node", result.output)

    def test_cml_wipe_node_not_stopped(self):
        with requests_mock.Mocker() as m:
            # Mock the request to return what we expect from the API.
            self.setup_mocks(m)
            virl = self.get_virl()
            runner = CliRunner()
            result = runner.invoke(virl, ["wipe", "node", "rtr-1"])
            self.assertEqual(1, result.exit_code)
            self.assertIn("is active", result.output)

    def test_cml_wipe_node_bogus_node(self):
        with requests_mock.Mocker() as m:
            # Mock the request to return what we expect from the API.
            self.setup_mocks(m)
            virl = self.get_virl()
            runner = CliRunner()
            result = runner.invoke(virl, ["wipe", "node", "rtr-3"])
            self.assertEqual(1, result.exit_code)
            self.assertIn("Node rtr-3 was not found in lab {}".format(self.get_test_id()), result.output)

    def test_cml_wipe_node_bogus_lab(self):
        src_dir = os.path.realpath(".virl")
        try:
            os.remove(".virl/current_cml_lab")
        except OSError:
            pass

        with open(".virl/cached_cml_labs/123456", "w") as fd:
            fd.write("lab: bogus\n")

        os.symlink("{}/cached_cml_labs/123456".format(src_dir), "{}/current_cml_lab".format(src_dir))
        with requests_mock.Mocker() as m:
            # Mock the request to return what we expect from the API.
            self.setup_mocks(m)
            virl = self.get_virl()
            runner = CliRunner()
            result = runner.invoke(virl, ["wipe", "node", "rtr-1"])
            os.remove(".virl/current_cml_lab")
            os.remove(".virl/cached_cml_labs/123456")
            self.assertEqual(1, result.exit_code)
            self.assertIn("Unable to find lab 123456", result.output)

    def test_cml_wipe_node_no_current_lab(self):
        try:
            os.remove(".virl/current_cml_lab")
        except OSError:
            pass

        with requests_mock.Mocker() as m:
            # Mock the request to return what we expect from the API.
            self.setup_mocks(m)
            virl = self.get_virl()
            runner = CliRunner()
            result = runner.invoke(virl, ["wipe", "node", "rtr-1"])
            self.assertEqual(1, result.exit_code)
            self.assertIn("No current lab set", result.output)
